#!/usr/bin/env python3

"""
Reveal on a filesystem

Based on the C++ code in
https://github.com/qt-creator/qt-creator/blob/master/src/plugins/coreplugin/fileutils.cpp, # noqa: E501
FileUtils::showInGraphicalShell
"""

import os
import platform

from qtpy import QtCore, QtWidgets


def _run_apple_script(command: str) -> None:
    script_args = ["-e", command]
    QtCore.QProcess.execute("/usr/bin/osascript", script_args)


def _run_apple_script_activate_application(app_name: str) -> None:
    _run_apple_script(f'tell application "{app_name}" to activate')


def _use_finder(file_info: QtCore.QFileInfo) -> None:
    app = "Finder"
    _run_apple_script(
        f'tell application "{app}" to reveal POSIX file '
        f'"{file_info.canonicalFilePath()}"'
    )
    _run_apple_script_activate_application(app)


def _use_explorer(file_info: QtCore.QFileInfo) -> None:
    explorer_path = QtCore.QStandardPaths.findExecutable("explorer.exe")
    script_args = [QtCore.QDir.toNativeSeparators(file_info.canonicalFilePath())]
    QtCore.QProcess.startDetached(explorer_path, script_args)


def _use_xdg_open(file_info: QtCore.QFileInfo) -> None:
    xdg_open_path = QtCore.QStandardPaths.findExecutable("xdg-open")
    if not xdg_open_path:
        QtWidgets.QMessageBox.critical(
            None,  # type: ignore
            "Cannot reveal path",
            "Unable to find the path to xdg-open",
        )
        return
    script_args = [file_info.canonicalFilePath()]
    QtCore.QProcess.startDetached(xdg_open_path, script_args)


# TODO: pathlib instead?
def reveal_on_filesystem(path: str) -> None:
    """
    Reveal the path in a filesystem GUI
    """
    file_info = QtCore.QFileInfo(path)
    if not file_info.exists():
        QtWidgets.QMessageBox.critical(
            None, "Cannot reveal path", f"Path '{path}' does not exist"  # type: ignore
        )
        return
    if platform.system() == "Darwin":
        _use_finder(file_info)
    elif platform.system() == "Windows":
        _use_explorer(file_info)
    elif platform.system() == "Linux":
        _use_xdg_open(file_info)
    else:
        raise RuntimeError(f"Unrecognised platform {platform.system()}")


def open_terminal_at(path: str) -> None:
    """
    Open a terminal at the given directory path.
    """
    # cannot use QProcess here, as it starts a shell in-pr5ocess, even if called
    # with startDetached
    file_info = QtCore.QFileInfo(path)
    if not file_info.exists():
        QtWidgets.QMessageBox.critical(
            None,  # type: ignore
            "Cannot open terminal at path",
            f"Path '{path}' does not exist",
        )
        return
    if platform.system() == "Darwin":
        app = "Terminal"
        _run_apple_script(
            f'tell application "{app}" to do script "cd " & '
            f'"{file_info.canonicalFilePath()}"'
        )
        _run_apple_script_activate_application(app)
    elif platform.system() == "Windows":
        os.system(
            "start /D "
            f"{QtCore.QDir.toNativeSeparators(file_info.canonicalFilePath())}"
        )
    elif platform.system() == "Linux":
        if [key for key in os.environ if "GNOME" in key]:
            # TODO: centos does complain that this is deprecated, and to use gio open
            # instead but am not sure how to detect that
            # trailing $SHELL is to keep the new terminal open
            os.system(
                "gnome-terminal -- /bin/bash -c "
                f"'cd {file_info.canonicalFilePath()}; $SHELL'"
            )
        else:
            QtWidgets.QMessageBox.critical(
                None,  # type: ignore
                "Cannot open terminal at path",
                "Unable to detect window manager",
            )
    else:
        raise RuntimeError(f"Unrecognised platform {platform.system()}")



#!/usr/bin/env python3

"""
Wizard introduction page
"""

from qtpy import QtWidgets


class LoadRecipeIntroPage(QtWidgets.QWizardPage):
    """
    Wizard page serving as an introduction to loading recipes.
    """

    def nextId(self) -> int:
        return -1 if self.wizard().has_load_errors else 1



#!/usr/bin/env python3

"""
Dialog for installing a new Conan config
"""

from functools import partial
import typing

from qtpy import QtCore, QtGui, QtWidgets, PYSIDE2

from cruiz.commands.context import ConanContext
from cruiz.interop.commandparameters import CommandParameters

if PYSIDE2:
    from cruiz.pyside2.local_cache_install_config import Ui_InstallConfigDialog

    QAction = QtWidgets.QAction
else:
    from cruiz.pyside6.local_cache_install_config import Ui_InstallConfigDialog

    QAction = QtGui.QAction

from cruiz.settings.managers.recentconanconfigs import (
    RecentConanConfigSettingsReader,
    RecentConanConfigSettings,
    RecentConanConfigSettingsWriter,
)
import cruiz.workers


class InstallConfigDialog(QtWidgets.QDialog):
    """
    Dialog for installing a new Conan config onto the local cache.
    """

    def __init__(self, context: ConanContext, parent: QtWidgets.QWidget) -> None:
        super().__init__(parent)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self._ui = Ui_InstallConfigDialog()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]
        self._context = context
        recent_config_paths_menu = QtWidgets.QMenu("Recent config paths", self)
        with RecentConanConfigSettingsReader() as settings:
            config_paths = settings.paths.resolve()
        if config_paths:
            for path in config_paths:
                path_action = QAction(path, self)
                path_action.triggered.connect(partial(self._set_url, path))
                recent_config_paths_menu.addAction(path_action)
        else:
            recent_config_paths_menu.setEnabled(False)
        self._ui.pathOrUrl.set_custom_menu(recent_config_paths_menu)
        self._ui.pathOrUrl.textChanged.connect(self._path_updated)
        self._ui.installButton.setEnabled(False)
        self._ui.installButton.clicked.connect(self._install)
        self._ui.buttonBox.button(QtWidgets.QDialogButtonBox.Cancel).clicked.connect(
            self._cancel
        )

    def _set_url(self, path: str) -> None:
        self._ui.pathOrUrl.setText(path)

    def _path_updated(self, path: str) -> None:
        self._ui.installButton.setEnabled(bool(path))

    def _install(self) -> None:
        self._ui.progressBar.setMaximum(0)
        params = CommandParameters("install_config", cruiz.workers.configinstall.invoke)
        named_args = params.named_arguments
        named_args["pathOrUrl"] = self._ui.pathOrUrl.text().strip()
        if self._ui.gitBranch.text():
            named_args["gitBranch"] = self._ui.gitBranch.text().strip()
        if self._ui.sourceFolder.text():
            named_args["sourceFolder"] = self._ui.sourceFolder.text().strip()
        if self._ui.targetFolder.text():
            named_args["targetFolder"] = self._ui.targetFolder.text().strip()
        self._context.install_config(params, self._install_complete)

    def _install_complete(self, result: typing.Any, exception: typing.Any) -> None:
        # pylint: disable=unused-argument
        self._ui.progressBar.setMaximum(1)
        if exception:
            QtWidgets.QMessageBox.critical(
                self,
                "Local cache config install failure",
                str(exception),
            )
            return
        with RecentConanConfigSettingsReader() as settings:
            config_paths = settings.paths.resolve()
        if self._ui.pathOrUrl.text() not in config_paths:
            settings = RecentConanConfigSettings()
            new_paths = config_paths + [self._ui.pathOrUrl.text()]
            settings.paths = new_paths  # type: ignore[assignment]
            RecentConanConfigSettingsWriter().sync(settings)
        self.accept()

    def _cancel(self) -> None:
        self._ui.progressBar.setMaximum(1)
        self._context.cancel()
        self.reject()



#!/usr/bin/env python3

"""
Formatting options to pass to Conan
"""

import typing


def format_options(options: typing.Dict[str, str]) -> typing.List[str]:
    """
    Format the dictionary of options appropriately for Conan
    """
    options_list = [f"{optkey}={optvalue}" for optkey, optvalue in options.items()]
    return options_list



#!/usr/bin/env python3

"""
Dialog to show cruiz information
"""

import sys
import typing

from qtpy import QtCore, QtGui, QtWidgets, PYSIDE2

from cruiz.version import get_version

if PYSIDE2:
    from cruiz.pyside2.about_dialog import Ui_AboutCruiz
else:
    from cruiz.pyside6.about_dialog import Ui_AboutCruiz


class AboutDialog(QtWidgets.QDialog):
    def __init__(self, parent: typing.Optional[QtWidgets.QWidget] = None) -> None:
        super().__init__(parent)
        self._ui = Ui_AboutCruiz()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]
        self._ui.cruiz.setPixmap(
            QtGui.QPixmap(":/cruiz.png").scaled(64, 64, QtCore.Qt.KeepAspectRatio)
        )
        self._ui.version.setText(f"Version: {get_version()}")
        self._ui.python.setText(f"Python executable: {sys.executable}")
        self._ui.python_version.setText(f"Python version: {sys.version}")
        self._ui.pyside_version.setText(f"PySide version: {QtCore.__version__}")



#!/usr/bin/env python3

"""
Conan recipe find text dialog
"""

from qtpy import QtCore, QtWidgets, PYSIDE2

if PYSIDE2:
    from cruiz.pyside2.find_text_dialog import Ui_FindTextDialog
else:
    from cruiz.pyside6.find_text_dialog import Ui_FindTextDialog


class FindTextDialog(QtWidgets.QDialog):
    """
    Widget representing a Find dialog
    """

    search_forwards = QtCore.Signal(QtWidgets.QPlainTextEdit, str, bool, bool)
    search_backwards = QtCore.Signal(QtWidgets.QPlainTextEdit, str, bool, bool)

    def __init__(self, parent: QtWidgets.QWidget) -> None:
        super().__init__(parent)
        self._ui = Ui_FindTextDialog()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]
        find_next = QtWidgets.QPushButton("Find &next")
        find_prev = QtWidgets.QPushButton("Find &previous")
        find_next.clicked.connect(self._find_next)
        find_prev.clicked.connect(self._find_prev)
        self._ui.findTextButtonBox.addButton(
            find_next, QtWidgets.QDialogButtonBox.ActionRole
        )
        self._ui.findTextButtonBox.addButton(
            find_prev, QtWidgets.QDialogButtonBox.ActionRole
        )

    def _find_next(self) -> None:
        self.search_forwards.emit(
            self.parent(),
            self._ui.findTextSearch.text(),
            self._ui.findTextCaseSensitive.isChecked(),
            self._ui.findTextWraparound.isChecked(),
        )

    def _find_prev(self) -> None:
        self.search_backwards.emit(
            self.parent(),
            self._ui.findTextSearch.text(),
            self._ui.findTextCaseSensitive.isChecked(),
            self._ui.findTextWraparound.isChecked(),
        )



#!/usr/bin/env python3

"""
Child process commands
"""

from __future__ import annotations

import multiprocessing
import typing

from cruiz.interop.commandparameters import CommandParameters
from .utils import worker
from .utils.formatoptions import format_options
from .utils.message import Message, Success


def invoke(queue: multiprocessing.Queue[Message], params: CommandParameters) -> None:
    """
    Run 'conan test'
    """
    with worker.ConanWorker(queue, params) as api:
        assert params.recipe_path
        assert params.cwd
        assert params.package_reference
        assert params.profile
        args: typing.Dict[str, typing.Union[str, typing.List[str]]] = {
            "profile_names": [params.profile],
        }
        if params.options:
            args["options"] = format_options(params.options)
        if params.test_folder:
            args["test_build_folder"] = str(params.test_folder)

        result = api.test(
            params.recipe_path,
            params.package_reference,
            **args,
        )

        queue.put(Success(result))



#!/usr/bin/env python3

"""
Settings context manager for recent Conan configurations
"""

import typing

from .basesettings import BaseSettings, ListSetting, CommonSettings, SettingMeta
from .writermixin import _WriterMixin
from .valueclasses import ListValue


# TODO: CommonSettings isn't completely used
class RecentConanConfigSettings(CommonSettings):
    """
    Settings representing the recent Conan configurations visited
    """

    def __init__(self) -> None:
        self._property_meta = {
            "paths": SettingMeta("RecentConanConfigs", ListSetting, [], ListValue),
        }

    @property
    def paths(self) -> ListSetting:
        """
        Get the paths of the Conan configurations visited
        """
        result: typing.List[str] = []
        assert self.settings_reader
        settings = self.settings_reader.settings
        # TODO: remove count on the reader, and replace by
        # num child groups / keys per entry in array
        assert self.settings_reader.count == (
            len(self.settings_reader.settings.childGroups()) / 1
        )
        for i in range(self.settings_reader.count):
            settings.setArrayIndex(i)
            result.append(str(settings.value("Path")))
        return ListSetting(result, [])

    @paths.setter
    def paths(self, paths: typing.List[str]) -> None:
        """
        Set the list of paths used for Conan configurations.
        """
        # avoids private name mangling
        setattr(  # noqa: B010
            self, "__new_paths", ListValue(paths, "paths", "RecentConanConfigs", "Path")
        )


class RecentConanConfigSettingsReader:
    """
    Context manager to read recent Conan configurations from disk
    """

    def __init__(self) -> None:
        self.group = ""
        self.array = "RecentConanConfigs"
        self.settings = BaseSettings.make_settings()
        self.count = 0

    def __enter__(self) -> RecentConanConfigSettings:
        self.count = self.settings.beginReadArray(self.array)
        self._settings_object = RecentConanConfigSettings()
        self._settings_object.settings_reader = self
        return self._settings_object

    def __exit__(
        self, exc_type: typing.Any, exc_value: typing.Any, exc_traceback: typing.Any
    ) -> typing.Any:
        self.settings.endArray()
        self._settings_object.settings_reader = None
        del self._settings_object
        if exc_type is None:
            pass
        else:
            # propagate exception
            return False


class RecentConanConfigSettingsWriter(_WriterMixin):
    """
    Utility class to write recent Conan configurations to disk
    """

    def __init__(self) -> None:
        self._reader_for_writer = RecentConanConfigSettingsReader()


class RecentConanConfigSettingsDeleter:
    """
    Utility class to delete recent Conan configurations from disk settings
    """

    def __init__(self) -> None:
        self._reader_for_deleter = RecentConanConfigSettingsReader()

    def delete(self, path: str) -> None:
        """
        Delete the specified Conan configuration from disk settings
        """
        with self._reader_for_deleter as settings:
            current_list = settings.paths.resolve()
        current_list = [x for x in current_list if x != path]
        BaseSettings.make_settings().remove(self._reader_for_deleter.array)
        if current_list:
            with BaseSettings.WriteArray(
                self._reader_for_deleter.array, replace=True
            ) as settings:
                for i, path in enumerate(current_list):
                    settings.setArrayIndex(i)
                    settings.setValue("Path", path)



#!/usr/bin/env python3

"""
Child process commands
"""

from __future__ import annotations

import multiprocessing
import os
import urllib.parse
import typing

from cruiz.interop.commandparameters import CommandParameters
from cruiz.interop.pod import ConanRemote, ConanHook
from .utils import worker, message


# pylint: disable=protected-access, import-outside-toplevel


# copied from distutils.url.strtobool and modified
def _strtobool(val: str) -> bool:
    """Convert a string representation of truth to true (1) or false (0).

    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
    val = val.lower()
    if val in ("y", "yes", "t", "true", "on", "1"):
        return True
    if val in ("n", "no", "f", "false", "off", "0"):
        return False
    raise ValueError("invalid truth value %r" % (val,))


def _remotes_list(api: typing.Any) -> typing.List[ConanRemote]:
    result = api.remote_list()
    try:
        # conan 1.19+
        result2 = [
            ConanRemote(remote.name, remote.url, not remote.disabled)
            for remote in result
        ]
    except AttributeError:
        result2 = [ConanRemote(remote.name, remote.url, True) for remote in result]
    return result2


def _remotes_sync(api: typing.Any, remotes: typing.List[str]) -> None:
    for remote in _remotes_list(api):
        api.remote_remove(remote.name)
    for remote_str in remotes:
        remote = ConanRemote.from_string(remote_str)
        api.remote_add(remote.name, remote.url)
        api.remote_set_disabled_state(remote.name, not remote.enabled)


def _profiles_dir(api: typing.Any) -> str:
    # pylint: disable=pointless-statement
    try:
        # Conan 1.18+
        profile_dir = api.app.cache.profiles_path
        if not os.path.isdir(profile_dir):
            # this creates the default profile
            api.app.cache.default_profile
    except AttributeError:
        profile_dir = api._cache.profiles_path
        if not os.path.isdir(profile_dir):
            # this creates the default profile
            api._cache.default_profile
    return profile_dir


def _default_profile_path(api: typing.Any) -> str:
    try:
        # Conan 1.18+
        api.create_app()
        default_profile_path = api.app.cache.default_profile_path
    except AttributeError:
        api.invalidate_caches()
        default_profile_path = api._cache.default_profile_path
    return default_profile_path


def _profile_meta(
    api: typing.Any, profile: str
) -> typing.Dict[str, typing.Dict[str, typing.Any]]:
    result = api.read_profile(profile)
    details: typing.Dict[str, typing.Dict[str, typing.Any]] = {"settings": {}}
    for key, value in result.settings.items():
        details["settings"][key] = value
    return details


def _conan_version() -> str:
    import conans

    return conans.__version__


def _get_package_layout(
    api: typing.Any, reference: str, short_paths: bool
) -> typing.Tuple[typing.Any, typing.Any]:
    from conans.model.ref import ConanFileReference

    file_ref = ConanFileReference.loads(reference)
    try:
        # Conan 1.18+
        layout = api.app.cache.package_layout(file_ref, short_paths=short_paths)
    except AttributeError:
        layout = api._cache.package_layout(file_ref, short_paths=short_paths)
    return layout, file_ref


def _package_dir(
    api: typing.Any,
    reference: str,
    package_id: str,
    package_revision: str,
    short_paths: bool,
) -> str:
    from conans.model.ref import PackageReference

    layout, file_ref = _get_package_layout(api, reference, short_paths)
    pref = PackageReference(file_ref, package_id, package_revision)
    package_dir = layout.package(pref)
    return package_dir


def _package_export_dir(api: typing.Any, reference: str, short_paths: bool) -> str:
    layout, _ = _get_package_layout(api, reference, short_paths)
    package_export_dir = layout.export()
    return package_export_dir


def _package_export_sources_dir(
    api: typing.Any, reference: str, short_paths: bool
) -> str:
    layout, _ = _get_package_layout(api, reference, short_paths)
    package_export_sources_dir = layout.export_sources()
    return package_export_sources_dir


def _editable_list(api: typing.Any) -> typing.List[str]:
    result = api.editable_list()
    return result


def _editable_add(api: typing.Any, ref: str, path: str) -> typing.Any:
    result = api.editable_add(
        path,
        ref,
        None,  # standard layout - use recipe dir to write package to
        None,  # cwd, fine to be None as the absolute path to the recipe is provided
    )
    return result


def _editable_remove(api: typing.Any, ref: str) -> typing.Any:
    result = api.editable_remove(ref)
    return result


def _inspect_recipe(api: typing.Any, recipe_path: str) -> typing.Dict[str, typing.Any]:
    result = api.inspect(recipe_path, None)  # get all attributes
    return result


def _hook_path(api: typing.Any) -> str:
    try:
        # conan 1.18+
        result = api.app.cache.hooks_path
    except AttributeError:
        result = api._cache.hooks_path
    return result


def _enabled_hooks(api: typing.Any) -> bool:
    try:
        # conan 1.18+
        result = api.app.cache.config.hooks
    except AttributeError:
        result = api._cache.config.hooks
    return result


def _available_hooks(api: typing.Any) -> typing.List[str]:
    try:
        # conan 1.18+
        hooks_dir = api.app.cache.hooks_path
    except AttributeError:
        hooks_dir = api._cache.hooks_path
    if not hooks_dir or not os.path.isdir(hooks_dir):
        return []
    hook_files = []
    for root, dirs, files in os.walk(hooks_dir):
        if ".git" in dirs:
            # ignore .git folders
            dirs.remove(".git")
        for file in files:
            if file.endswith(".py"):
                full_path = os.path.join(root, file)
                stored_path = os.path.relpath(full_path, hooks_dir)
                hook_files.append(stored_path)
    return hook_files


def _hooks_get(api: typing.Any) -> typing.List[ConanHook]:
    try:
        # conan 1.18+
        hooks_dir = api.app.cache.hooks_path
        enabled_hooks = api.app.cache.config.hooks
    except AttributeError:
        hooks_dir = api._cache.hooks_path
        enabled_hooks = api._cache.config.hooks
    if not hooks_dir or not os.path.isdir(hooks_dir):
        return []
    hook_files: typing.List[ConanHook] = []
    for root, dirs, files in os.walk(hooks_dir):
        if ".git" in dirs:
            # ignore .git folders
            dirs.remove(".git")
        for file in files:
            if file.endswith(".py"):
                full_path = os.path.join(root, file)
                stored_path = os.path.relpath(full_path, hooks_dir)
                hook_enabled = (
                    file in enabled_hooks or os.path.splitext(file)[0] in enabled_hooks
                )
                hook_files.append(ConanHook(stored_path, hook_enabled))
    return hook_files


def _hooks_sync(api: typing.Any, hook_changes: typing.List[str]) -> None:
    for change in hook_changes:
        hook = ConanHook.from_string(change)
        hook_config = f"hooks.{hook.path}"
        if hook.enabled:
            _set_config(api, hook_config, None)
        else:
            if _has_config_option(api, "hooks", hook.path):
                _rm_config(api, hook_config)
            else:
                no_ext_path = os.path.splitext(hook.path)[0]
                hook_config = f"hooks.{no_ext_path}"
                assert _has_config_option(api, "hooks", no_ext_path)
                _rm_config(api, hook_config)


def _enable_hook(api: typing.Any, hook: str, hook_enabled: bool) -> None:
    hook = f"hooks.{hook}"
    if hook_enabled:
        _set_config(api, hook, None)
    else:
        _rm_config(api, hook)


def _get_conandata(api: typing.Any, recipe_path: str) -> typing.Dict[str, typing.Any]:
    try:
        # conan 1.18+
        result = api.app.loader._load_data(recipe_path)
    except AttributeError:
        result = api._cache.loader._load_data(recipe_path)
    return result


def _get_config(api: typing.Any, key: str) -> typing.Optional[str]:
    # pylint: disable=broad-except
    try:
        try:
            # conan 1.18+
            result = api.app.cache.config.get_item(key)
        except AttributeError:
            result = api._cache.config.get_item(key)
        return result
    except Exception:
        return None


def _set_config(api: typing.Any, key: str, value: typing.Optional[str]) -> None:
    try:
        # conan 1.18+
        api.app.cache.config.set_item(key, value)
    except AttributeError:
        api._cache.config.set_item(key, value)


def _rm_config(api: typing.Any, key: str) -> None:
    try:
        # conan 1.18+
        api.app.cache.config.rm_item(key)
    except AttributeError:
        api._cache.config.rm_item(key)


def _has_config_option(api: typing.Any, key: str, value: str) -> bool:
    try:
        # conan 1.18+
        return api.app.cache.config.has_option(key, value)
    except AttributeError:
        return api._cache.config.has_option(key, value)


def _get_config_envvars(api: typing.Any) -> typing.Dict[str, str]:
    try:
        # conan 1.18+
        return api.app.cache.config.env_vars
    except AttributeError:
        return api._cache.config.env_vars


# pylint: disable=too-many-statements
def invoke(
    request_queue: multiprocessing.JoinableQueue[str],
    reply_queue: multiprocessing.Queue[message.Message],
    params: CommandParameters,
) -> None:
    # pylint: disable=too-many-branches
    """
    Run continuous loop, waiting on requests from the main process
    """
    with worker.ConanWorker(reply_queue, params) as api:
        while True:
            try:
                request = request_queue.get()
                if request == "end":
                    request_queue.task_done()
                    break
                if "?" in request:
                    split = request.split("?")
                    request = split[0]
                    request_params = urllib.parse.parse_qs(split[1])
                if request == "remotes_list":
                    result = _remotes_list(api)
                elif request == "remotes_sync":
                    _remotes_sync(api, request_params["remotes"])
                    result = None
                elif request == "profiles_dir":
                    result = _profiles_dir(api)  # type: ignore[assignment]
                elif request == "default_profile_path":
                    result = _default_profile_path(api)  # type: ignore[assignment]
                elif request == "profile_meta":
                    result = _profile_meta(
                        api, request_params["name"][0]
                    )  # type: ignore[assignment]
                elif request == "version":
                    result = _conan_version()  # type: ignore[assignment]
                elif request == "package_dir":
                    result = _package_dir(  # type: ignore[assignment]
                        api,
                        request_params["ref"][0],
                        request_params["package_id"][0],
                        request_params["revision"][0],
                        _strtobool(request_params["short_paths"][0]),
                    )
                elif request == "package_export_dir":
                    result = _package_export_dir(  # type: ignore[assignment]
                        api,
                        request_params["ref"][0],
                        _strtobool(request_params["short_paths"][0]),
                    )
                elif request == "package_export_sources_dir":
                    result = _package_export_sources_dir(  # type: ignore[assignment]
                        api,
                        request_params["ref"][0],
                        _strtobool(request_params["short_paths"][0]),
                    )
                elif request == "editable_list":
                    result = _editable_list(api)  # type: ignore[assignment]
                elif request == "editable_add":
                    result = _editable_add(
                        api, request_params["ref"][0], request_params["path"][0]
                    )
                elif request == "editable_remove":
                    result = _editable_remove(api, request_params["ref"][0])
                elif request == "inspect_recipe":
                    result = _inspect_recipe(
                        api, request_params["path"][0]
                    )  # type: ignore[assignment]
                elif request == "hook_path":
                    result = _hook_path(api)  # type: ignore[assignment]
                elif request == "enabled_hooks":
                    result = _enabled_hooks(api)  # type: ignore[assignment]
                elif request == "available_hooks":
                    result = _available_hooks(api)  # type: ignore[assignment]
                elif request == "get_hooks":
                    result = _hooks_get(api)  # type: ignore[assignment]
                elif request == "hooks_sync":
                    _hooks_sync(api, request_params["hooks"])
                    result = None
                elif request == "enable_hook":
                    _enable_hook(
                        api,
                        request_params["hook"][0],
                        _strtobool(request_params["hook_enabled"][0]),
                    )
                    result = None
                elif request == "get_cmake_generator":
                    result = _get_config(
                        api, "general.cmake_generator"
                    )  # type: ignore[assignment]
                elif request == "get_conandata":
                    result = _get_conandata(
                        api, request_params["path"][0]
                    )  # type: ignore[assignment]
                elif request == "get_config":
                    result = _get_config(
                        api, request_params["config"][0]
                    )  # type: ignore[assignment]
                elif request == "set_config":
                    _set_config(
                        api,
                        request_params["config"][0],
                        request_params["value"][0],
                    )
                    result = None
                elif request == "get_config_envvars":
                    result = _get_config_envvars(api)  # type: ignore[assignment]
                reply_queue.put(message.Success(result))
                request_queue.task_done()
                # ensure that the result doesn't accidentally appear in
                # subsequent loop iterations
                del result
            # pylint: disable=broad-except
            except Exception as exception:
                reply_queue.put(message.Failure(Exception(str(exception))))
                request_queue.task_done()
    request_queue.join()
    request_queue.close()
    request_queue.join_thread()



#!/usr/bin/env python3

"""
Child process commands
"""

from __future__ import annotations

import multiprocessing
import typing

from cruiz.interop.commandparameters import CommandParameters
from .utils import worker
from .utils.formatoptions import format_options
from .utils.message import Message, Success


def invoke(queue: multiprocessing.Queue[Message], params: CommandParameters) -> None:
    """
    Run 'conan install'
    """
    with worker.ConanWorker(queue, params) as api:
        assert params.recipe_path
        assert params.cwd
        assert params.profile
        args: typing.Dict[str, typing.Union[bool, str, typing.List[str]]] = {
            "path": str(params.recipe_path),
            "profile_names": [params.profile],
        }
        if params.version:
            args["version"] = params.version
        if params.install_folder:
            args["install_folder"] = str(params.install_folder)
        if params.options:
            args["options"] = format_options(params.options)
        if params.user:
            args["user"] = params.user
        if params.channel:
            args["channel"] = params.channel
        if "--update" in params.arguments:
            args["update"] = True

        result = api.install(**args)
        worker.replace_conan_version_struct_with_string(result)
        queue.put(Success(result))



#!/usr/bin/env python3

"""
Settings context managers for conan preferences
"""

import typing

from .basesettings import (
    BaseSettings,
    StringSetting,
    ComparableCommonSettings,
    SettingMeta,
)
from .writermixin import _WriterMixin
from .valueclasses import ScalarValue


class ConanSettings(ComparableCommonSettings):
    """
    Representation of Conan settings on disk
    """

    def __init__(self) -> None:
        self._property_meta = {
            "log_level": SettingMeta(
                "Log/Level", StringSetting, "CRITICAL", ScalarValue
            ),
            "conandata_version_yaml_pathsegment": SettingMeta(
                "ConanData/VersionExtractionPathSegment",
                StringSetting,
                "sources",
                ScalarValue,
            ),
        }

    @property
    def log_level(self) -> StringSetting:
        """
        Get the logging level used for Conan
        """
        return self._get_value_via_meta()

    @log_level.setter
    def log_level(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conandata_version_yaml_pathsegment(self) -> StringSetting:
        """
        Get the YAML segment used to reference the 'version' element in conandata.yml
        """
        return self._get_value_via_meta()

    @conandata_version_yaml_pathsegment.setter
    def conandata_version_yaml_pathsegment(self, value: str) -> None:
        self._set_value_via_meta(value)


class ConanSettingsReader:
    """
    Context manager for reading Conan settings from disk
    """

    def __init__(self) -> None:
        self.settings = BaseSettings.make_settings()
        self.group = "Conan"

    def __enter__(self) -> ConanSettings:
        self.settings.beginGroup(self.group)
        self._settings_object = ConanSettings()
        self._settings_object.settings_reader = self
        return self._settings_object

    def __exit__(
        self, exc_type: typing.Any, exc_value: typing.Any, exc_traceback: typing.Any
    ) -> typing.Any:
        self.settings.endGroup()
        self._settings_object.settings_reader = None
        del self._settings_object
        if exc_type is None:
            pass
        else:
            # propagate exception
            return False


class ConanSettingsWriter(_WriterMixin):
    """
    Utility class for writing changed Conan settings to disk
    """

    def __init__(self) -> None:
        self._reader_for_writer = ConanSettingsReader()



#!/usr/bin/env python3

"""
Stream data via a multiprocessing queue
"""

from __future__ import annotations

import multiprocessing
import six
import typing

from .colorarma_conversion import convert_from_colorama_to_html

from cruiz.workers.utils.message import Message


if False:
    import io

    # Conan 1.30.0+ changed the assumptions on the base-class of streams used in their
    # runner output to be based from six, so this Python 3 implementation is no longer
    # valid it may come back in future Conan versions that are Python 3 only
    class QueuedStreamPy3(io.RawIOBase):
        """
        A stream class that uses multiprocessing.Queue to send messages
        """

        def __init__(self, queue: multiprocessing.Queue[Message], message_type):
            super().__init__()
            self._queue = queue
            self._message_type = message_type

        # configure the IOBase
        def seekable(self):
            """
            Stream is not seekable
            """
            return False

        def writeable(self):
            """
            Stream is writeable
            """
            # pylint: disable=no-self-use
            return True

        def readable(self):
            """
            Stream is not readable
            """
            return False

        def isatty(self):
            """
            Stream is not interactive
            """
            # TODO: could this be True, which asks the queue for info?
            return False

        # implement functions of interest
        def write(self, message):
            """
            Write a message.
            """
            lines = message.split("\n")
            for line in lines:
                if not line:
                    continue
                self._queue.put(self._message_type(convert_from_colorama_to_html(line)))

        def flush(self):
            """
            Flush the stream.
            """


class QueuedStreamSix(six.StringIO):
    """
    A stream class that uses multiprocessing.Queue to send messages.
    Complete messages may be passed piecemeal, so that coloured output is generated.
    This buffers up a message, so that it is passed across the process divide as a
    single HTML paragraph.
    """

    def __init__(
        self, queue: multiprocessing.Queue[Message], message_type: typing.Any
    ) -> None:
        super().__init__()
        self._queue = queue
        self._message_type = message_type
        self._block = six.StringIO()

    def write(self, message: str) -> int:
        """
        Write a message.
        """
        result = self._block.write(convert_from_colorama_to_html(message))
        if message.endswith("\n"):
            self._queue.put(self._message_type(self._block.getvalue()))
            self._block = six.StringIO()
        return result



#!/usr/bin/env python3

"""
Restart the whole application
"""

from qtpy import QtCore


def restart_cruiz() -> None:
    """
    Quit the application and restart it.
    """
    qApp.quit()  # type: ignore[name-defined] # noqa: F821
    QtCore.QProcess.startDetached(
        qApp.arguments()[0], qApp.arguments()  # type: ignore[name-defined] # noqa: F821
    )



#!/usr/bin/env python3

"""
Utility to save and restore the os.environ temporarily using a with statement.
"""

import os
import typing


class EnvironSaver:
    """
    Use a 'with' statement to limit the scope of these changes
    """

    def __init__(self) -> None:
        self._saved: typing.Optional[typing.Dict[str, str]] = None

    def __enter__(self) -> None:
        self._saved = os.environ.copy()

    def __exit__(
        self, exc_type: typing.Any, exc_value: typing.Any, exc_traceback: typing.Any
    ) -> None:
        self.restore()

    def restore(self) -> None:
        """
        Restore os.environ to that saved
        """
        os.environ.clear()
        # TODO: want to use this next statement, but mypy has an issue with it
        # os.environ.update(self._saved)
        assert self._saved
        for key, value in self._saved.items():
            os.environ[key] = value



#!/usr/bin/env python3

"""
Enumeration for each binary type
"""

from enum import IntFlag
from enum import auto


class ConanPackageTypeFlags(IntFlag):
    """
    Flags for each binary type of Conan package
    """

    BUILDABLE = auto()
    BINARY = auto()
    EDITABLE = auto()



#!/usr/bin/env python3

"""
Dialog for removing an environment variable
"""

from qtpy import QtCore, QtWidgets, PYSIDE2

if PYSIDE2:
    from cruiz.pyside2.local_cache_remove_environment import Ui_RemoveEnvironmentDialog
else:
    from cruiz.pyside6.local_cache_remove_environment import Ui_RemoveEnvironmentDialog


class RemoveEnvironmentDialog(QtWidgets.QDialog):
    """
    Dialog for removing an environment variable from the collection used to run
    Conan commands
    """

    def __init__(self, parent: QtWidgets.QWidget) -> None:
        super().__init__(parent)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self._ui = Ui_RemoveEnvironmentDialog()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]
        self._ui.name.textChanged.connect(self._updated)
        self._ui.buttonBox.button(QtWidgets.QDialogButtonBox.Ok).setEnabled(False)
        self.name: str = ""

    def _set_name(self) -> None:
        self._ui.name.setText(self.sender().text())

    def _updated(self) -> None:
        self._ui.buttonBox.button(QtWidgets.QDialogButtonBox.Ok).setEnabled(
            bool(self._ui.name.text())
        )

    def accept(self) -> None:
        self.name = self._ui.name.text()
        super().accept()



#!/usr/bin/env python3

"""
Resource generation from PySide
"""

import logging
import os
import pathlib
import platform
import subprocess
import typing

from qtpy import PYSIDE2

if PYSIDE2:
    SUBDIR = "pyside2"
    RCC = "pyside2-rcc"
    UIC = "pyside2-uic"
else:
    SUBDIR = "pyside6"
    RCC = "pyside6-rcc"
    UIC = "pyside6-uic"


logger = logging.getLogger(__name__)


def _run_command_if_out_of_date(
    input: pathlib.Path, output: pathlib.Path, cmd_args: typing.List[str]
) -> bool:
    if output.exists():
        input_mtime = os.path.getmtime(input)
        output_mtime = os.path.getmtime(output)
        if input_mtime < output_mtime:
            return False
    # compile out of date resources
    logger.debug("Running: '%s'", " ".join(cmd_args))
    if platform.system() == "Windows":
        subprocess.check_call(
            cmd_args,
            creationflags=subprocess.CREATE_NO_WINDOW,  # type: ignore[attr-defined]
        )
    else:
        subprocess.check_call(cmd_args)
    return True


def _ensure_resource_file_is_up_to_date(
    dst_dir: pathlib.Path, resources_dir: pathlib.Path
) -> None:
    resource_file = resources_dir / "cruizres.qrc"
    compiled_rcc_file = dst_dir / "resources.py"
    compile_args = [
        RCC,
        "-o",
        str(compiled_rcc_file),
        str(resource_file),
    ]
    _run_command_if_out_of_date(resource_file, compiled_rcc_file, compile_args)


def _ensure_ui_files_are_up_to_date(
    dst_dir: pathlib.Path, resources_dir: pathlib.Path
) -> None:
    for ui_file in resources_dir.glob("*.ui"):
        compiled_ui_file = dst_dir.joinpath(ui_file.name).with_suffix(".py")
        compile_args = [
            UIC,
            "-o",
            str(compiled_ui_file),
            str(ui_file),
        ]
        _run_command_if_out_of_date(ui_file, compiled_ui_file, compile_args)


def generate_resources() -> None:
    """
    Generate PySide resources if out of date
    """
    current_dir = pathlib.Path(__file__).parent
    resources_dir = current_dir / "resources"
    if not resources_dir.exists():
        # resources directory does not exist in binary distributions, so early out
        logger.debug("Resource directory does not exist; compiled resources are fixed")
        return
    logger.debug("Checking for out-of-date resources...")
    _ensure_resource_file_is_up_to_date(current_dir / SUBDIR, resources_dir)
    _ensure_ui_files_are_up_to_date(current_dir / SUBDIR, resources_dir)



#!/usr/bin/env python3

"""
Settings context manager value classes
"""

from dataclasses import dataclass
import typing


@dataclass
class ScalarValue:
    """
    Instance representing a scalar value of any type that can be
    queried via properties of a *Settings class.
    """

    value: typing.Any
    property_name: str
    key: str


@dataclass
class ListValue:
    """
    Instance representing a list of any values that can be
    queried via properties of a *Settings class.

    The writer mixin assumes the whole list is provided each time it needs to be
    serialised, since there is no other way to determine if an entry needs to be
    removed.
    """

    value: typing.List[typing.Any]
    property_name: str
    key: str
    entry_key: str


@dataclass
class DictValue:
    """
    Instance representing a dictionary of any string key and value pairs that can be
    queried via properties of a *Settings class.

    If any keys have a None value, these are removed from the serialised settings
    by the writer mixin.
    """

    value: typing.Any
    property_name: str
    key: str
    entry_key: str
    entry_value: str



#!/usr/bin/env python3

"""
Wizard page for selecting the initial profile
"""

import pathlib
import typing

from qtpy import QtWidgets

from cruiz.commands.context import managed_conan_context
from cruiz.commands.logdetails import LogDetails

from cruiz.widgets.util import BlockSignals
from cruiz.settings.managers.recipe import RecipeSettingsReader


class LoadRecipeInitialProfilePage(QtWidgets.QWizardPage):
    """
    Wizard page allowing selection of an initial profile to apply to the recipe.
    """

    @property
    def _ui(self) -> typing.Any:
        return self.wizard().ui

    def nextId(self) -> int:
        return -1

    def initializePage(self) -> None:
        self.registerField(
            "initial_profile*", self._ui.initial_profile, "currentText"  # type: ignore
        )
        self._ui.initial_profile.currentIndexChanged.connect(
            self._on_initial_profile_changed
        )
        with BlockSignals(self._ui.initial_profile) as blocked_widget:
            blocked_widget.clear()
            for profile_path in self._get_profile_list():
                blocked_widget.addItem(str(profile_path))
            blocked_widget.setCurrentIndex(-1)
        if self.wizard().original_recipe:
            with RecipeSettingsReader.from_recipe(
                self.wizard().original_recipe
            ) as settings:
                original_profile = settings.profile.resolve()
            self._ui.initial_profile.setCurrentText(original_profile)
        super().initializePage()

    def cleanupPage(self) -> None:
        self._ui.initial_profile.currentIndexChanged.disconnect()
        return super().cleanupPage()

    def _on_initial_profile_changed(self, index: int) -> None:
        # pylint: disable=unused-argument
        self.completeChanged.emit()

    def _get_profile_list(self) -> typing.List[pathlib.Path]:
        self._ui.initial_profile_log.hide()
        log_details = LogDetails(self._ui.initial_profile_log, None, True, False, None)
        log_details.logging.connect(self._on_get_profile_list_output)
        with managed_conan_context(
            self._ui.local_cache_name.currentText(), log_details
        ) as context:
            profile_list = context.get_list_of_profiles()
        return [p[0] for p in profile_list]

    def _on_get_profile_list_output(self) -> None:
        self._ui.initial_profile_log.show()



#!/usr/bin/env python3

"""
Widget debugging
"""

import logging

from qtpy import QtCore, QtWidgets, PYSIDE2


def _widget_window_flags_to_string(widget: QtWidgets.QWidget) -> str:
    flags = widget.windowFlags()
    text = ""
    # window type
    window_type = flags & QtCore.Qt.WindowType_Mask
    if window_type == QtCore.Qt.Window:
        text += "Window"
    if window_type == QtCore.Qt.Dialog:
        text += "Dialog"
    if window_type == QtCore.Qt.Sheet:
        text += "Sheet"
    if window_type == QtCore.Qt.Drawer:
        text += "Drawer"
    if window_type == QtCore.Qt.Popup:
        text += "Popup"
    if window_type == QtCore.Qt.Tool:
        text += "Tool"
    if window_type == QtCore.Qt.ToolTip:
        text += "ToolTip"
    if window_type == QtCore.Qt.SplashScreen:
        text += "SplashScreen"
    if window_type == QtCore.Qt.Desktop:
        text += "Desktop"
    if window_type == QtCore.Qt.SubWindow:
        text += "SubWindow"
    if window_type == QtCore.Qt.ForeignWindow:
        text += "ForeignWindow"
    if window_type == QtCore.Qt.CoverWindow:
        text += "CoverWindow"
    # flags
    if flags & QtCore.Qt.MSWindowsFixedSizeDialogHint:
        text += ":MSWindowsFixedSizeDialogHint"
    if flags & QtCore.Qt.MSWindowsOwnDC:
        text += ":MSWindowsOwnDC"
    if flags & QtCore.Qt.BypassWindowManagerHint:
        text += ":BypassWindowManagerHint"
    if flags & QtCore.Qt.X11BypassWindowManagerHint:
        text += ":X11BypassWindowManagerHint"
    if flags & QtCore.Qt.FramelessWindowHint:
        text += ":FramelessWindowHint"
    if flags & QtCore.Qt.NoDropShadowWindowHint:
        text += ":NoDropShadowWindowHint"
    if flags & QtCore.Qt.CustomizeWindowHint:
        text += ":CustomizeWindowHint"
    if flags & QtCore.Qt.WindowTitleHint:
        text += ":WindowTitleHint"
    if flags & QtCore.Qt.WindowSystemMenuHint:
        text += ":WindowSystemMenuHint"
    if flags & QtCore.Qt.WindowMinimizeButtonHint:
        text += ":WindowMinimizeButtonHint"
    if flags & QtCore.Qt.WindowMaximizeButtonHint:
        text += ":WindowMaximizeButtonHint"
    if flags & QtCore.Qt.WindowMinMaxButtonsHint:
        text += ":WindowMinMaxButtonsHint"
    if flags & QtCore.Qt.WindowCloseButtonHint:
        text += ":WindowCloseButtonHint"
    if flags & QtCore.Qt.WindowContextHelpButtonHint:
        text += ":WindowContextHelpButtonHint"
    if flags & QtCore.Qt.MacWindowToolBarButtonHint:
        text += ":MacWindowToolBarButtonHint"
    if flags & QtCore.Qt.WindowFullscreenButtonHint:
        text += ":WindowFullscreenButtonHint"
    if flags & QtCore.Qt.BypassGraphicsProxyWidget:
        text += ":BypassGraphicsProxyWidget"
    if flags & QtCore.Qt.WindowShadeButtonHint:
        text += ":WindowShadeButtonHint"
    if flags & QtCore.Qt.WindowStaysOnTopHint:
        text += ":WindowStaysOnTopHint"
    if flags & QtCore.Qt.WindowStaysOnBottomHint:
        text += ":WindowStaysOnBottomHint"
    if flags & QtCore.Qt.WindowTransparentForInput:
        text += ":WindowTransparentForInput"
    if flags & QtCore.Qt.WindowOverridesSystemGestures:
        text += ":WindowOverridesSystemGestures"
    if flags & QtCore.Qt.WindowDoesNotAcceptFocus:
        text += ":WindowDoesNotAcceptFocus"
    if flags & QtCore.Qt.MaximizeUsingFullscreenGeometryHint:
        text += ":MaximizeUsingFullscreenGeometryHint"
    return text


def _widget_attributes_to_string(widget: QtWidgets.QWidget) -> str:
    text = "Attrs"
    if widget.testAttribute(QtCore.Qt.WA_AcceptDrops):
        text += ":WA_AcceptDrops"
    if widget.testAttribute(QtCore.Qt.WA_AlwaysShowToolTips):
        text += ":WA_AlwaysShowToolTips"
    if PYSIDE2:
        if widget.testAttribute(QtCore.Qt.WA_ContentsPropagated):
            text += ":WA_ContentsPropagated"
    if widget.testAttribute(QtCore.Qt.WA_CustomWhatsThis):
        text += ":WA_CustomWhatsThis"
    if widget.testAttribute(QtCore.Qt.WA_DeleteOnClose):
        text += ":WA_DeleteOnClose"
    if widget.testAttribute(QtCore.Qt.WA_Disabled):
        text += ":WA_Disabled"
    if widget.testAttribute(QtCore.Qt.WA_DontShowOnScreen):
        text += ":WA_DontShowOnScreen"
    if widget.testAttribute(QtCore.Qt.WA_ForceDisabled):
        text += ":WA_ForceDisabled"
    if widget.testAttribute(QtCore.Qt.WA_ForceUpdatesDisabled):
        text += ":WA_ForceUpdatesDisabled"
    if PYSIDE2:
        if widget.testAttribute(QtCore.Qt.WA_GroupLeader):
            text += ":WA_GroupLeader"
    if widget.testAttribute(QtCore.Qt.WA_Hover):
        text += ":WA_Hover"
    if widget.testAttribute(QtCore.Qt.WA_InputMethodEnabled):
        text += ":WA_InputMethodEnabled"
    if widget.testAttribute(QtCore.Qt.WA_KeyboardFocusChange):
        text += ":WA_KeyboardFocusChange"
    if widget.testAttribute(QtCore.Qt.WA_KeyCompression):
        text += ":WA_KeyCompression"
    if widget.testAttribute(QtCore.Qt.WA_LayoutOnEntireRect):
        text += ":WA_LayoutOnEntireRect"
    if widget.testAttribute(QtCore.Qt.WA_LayoutUsesWidgetRect):
        text += ":WA_LayoutUsesWidgetRect"
    if PYSIDE2:
        if widget.testAttribute(QtCore.Qt.WA_MacNoClickThrough):
            text += ":WA_MacNoClickThrough"
    if widget.testAttribute(QtCore.Qt.WA_MacOpaqueSizeGrip):
        text += ":WA_MacOpaqueSizeGrip"
    if widget.testAttribute(QtCore.Qt.WA_MacShowFocusRect):
        text += ":WA_MacShowFocusRect"
    if widget.testAttribute(QtCore.Qt.WA_MacNormalSize):
        text += ":WA_MacNormalSize"
    if widget.testAttribute(QtCore.Qt.WA_MacSmallSize):
        text += ":WA_MacSmallSize"
    if widget.testAttribute(QtCore.Qt.WA_MacMiniSize):
        text += ":WA_MacMiniSize"
    if PYSIDE2:
        if widget.testAttribute(QtCore.Qt.WA_MacVariableSize):
            text += ":WA_MacVariableSize"
        if widget.testAttribute(QtCore.Qt.WA_MacBrushedMetal):
            text += ":WA_MacBrushedMetal"
    if widget.testAttribute(QtCore.Qt.WA_Mapped):
        text += ":WA_Mapped"
    if widget.testAttribute(QtCore.Qt.WA_MouseNoMask):
        text += ":WA_MouseNoMask"
    if widget.testAttribute(QtCore.Qt.WA_MouseTracking):
        text += ":WA_MouseTracking"
    if widget.testAttribute(QtCore.Qt.WA_Moved):
        text += ":WA_Moved"
    if PYSIDE2:
        if widget.testAttribute(QtCore.Qt.WA_MSWindowsUseDirect3D):
            text += ":WA_MSWindowsUseDirect3D"
        if widget.testAttribute(QtCore.Qt.WA_NoBackground):
            text += ":WA_NoBackground"
    if widget.testAttribute(QtCore.Qt.WA_NoChildEventsForParent):
        text += ":WA_NoChildEventsForParent"
    if widget.testAttribute(QtCore.Qt.WA_NoChildEventsFromChildren):
        text += ":WA_NoChildEventsFromChildren"
    if widget.testAttribute(QtCore.Qt.WA_NoMouseReplay):
        text += ":WA_NoMouseReplay"
    if widget.testAttribute(QtCore.Qt.WA_NoMousePropagation):
        text += ":WA_NoMousePropagation"
    if widget.testAttribute(QtCore.Qt.WA_TransparentForMouseEvents):
        text += ":WA_TransparentForMouseEvents"
    if widget.testAttribute(QtCore.Qt.WA_NoSystemBackground):
        text += ":WA_NoSystemBackground"
    if widget.testAttribute(QtCore.Qt.WA_OpaquePaintEvent):
        text += ":WA_OpaquePaintEvent"
    if widget.testAttribute(QtCore.Qt.WA_OutsideWSRange):
        text += ":WA_OutsideWSRange"
    if widget.testAttribute(QtCore.Qt.WA_PaintOnScreen):
        text += ":WA_PaintOnScreen"
    if widget.testAttribute(QtCore.Qt.WA_PaintUnclipped):
        text += ":WA_PaintUnclipped"
    if widget.testAttribute(QtCore.Qt.WA_PendingMoveEvent):
        text += ":WA_PendingMoveEvent"
    if widget.testAttribute(QtCore.Qt.WA_PendingResizeEvent):
        text += ":WA_PendingResizeEvent"
    if widget.testAttribute(QtCore.Qt.WA_QuitOnClose):
        text += ":WA_QuitOnClose"
    if widget.testAttribute(QtCore.Qt.WA_Resized):
        text += ":WA_Resized"
    if widget.testAttribute(QtCore.Qt.WA_RightToLeft):
        text += ":WA_RightToLeft"
    if widget.testAttribute(QtCore.Qt.WA_SetCursor):
        text += ":WA_SetCursor"
    if widget.testAttribute(QtCore.Qt.WA_SetFont):
        text += ":WA_SetFont"
    if widget.testAttribute(QtCore.Qt.WA_SetPalette):
        text += ":WA_SetPalette"
    if widget.testAttribute(QtCore.Qt.WA_SetStyle):
        text += ":WA_SetStyle"
    if widget.testAttribute(QtCore.Qt.WA_ShowModal):
        text += ":WA_ShowModal"
    if widget.testAttribute(QtCore.Qt.WA_StaticContents):
        text += ":WA_StaticContents"
    if widget.testAttribute(QtCore.Qt.WA_StyleSheet):
        text += ":WA_StyleSheet"
    if widget.testAttribute(QtCore.Qt.WA_StyleSheetTarget):
        text += ":WA_StyleSheetTarget"
    if widget.testAttribute(QtCore.Qt.WA_TabletTracking):
        text += ":WA_TabletTracking"
    if widget.testAttribute(QtCore.Qt.WA_TranslucentBackground):
        text += ":WA_TranslucentBackground"
    if widget.testAttribute(QtCore.Qt.WA_UnderMouse):
        text += ":WA_UnderMouse"
    if widget.testAttribute(QtCore.Qt.WA_UpdatesDisabled):
        text += ":WA_UpdatesDisabled"
    if widget.testAttribute(QtCore.Qt.WA_WindowModified):
        text += ":WA_WindowModified"
    if widget.testAttribute(QtCore.Qt.WA_WindowPropagation):
        text += ":WA_WindowPropagation"
    if widget.testAttribute(QtCore.Qt.WA_MacAlwaysShowToolWindow):
        text += ":WA_MacAlwaysShowToolWindow"
    if widget.testAttribute(QtCore.Qt.WA_SetLocale):
        text += ":WA_SetLocale"
    if widget.testAttribute(QtCore.Qt.WA_StyledBackground):
        text += ":WA_StyledBackground"
    if widget.testAttribute(QtCore.Qt.WA_ShowWithoutActivating):
        text += ":WA_ShowWithoutActivating"
    if widget.testAttribute(QtCore.Qt.WA_NativeWindow):
        text += ":WA_NativeWindow"
    if widget.testAttribute(QtCore.Qt.WA_DontCreateNativeAncestors):
        text += ":WA_DontCreateNativeAncestors"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeDesktop):
        text += ":WA_X11NetWmWindowTypeDesktop"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeDock):
        text += ":WA_X11NetWmWindowTypeDock"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeToolBar):
        text += ":WA_X11NetWmWindowTypeToolBar"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeMenu):
        text += ":WA_X11NetWmWindowTypeMenu"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeUtility):
        text += ":WA_X11NetWmWindowTypeUtility"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeSplash):
        text += ":WA_X11NetWmWindowTypeSplash"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeDialog):
        text += ":WA_X11NetWmWindowTypeDialog"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeDropDownMenu):
        text += ":WA_X11NetWmWindowTypeDropDownMenu"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypePopupMenu):
        text += ":WA_X11NetWmWindowTypePopupMenu"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeToolTip):
        text += ":WA_X11NetWmWindowTypeToolTip"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeNotification):
        text += ":WA_X11NetWmWindowTypeNotification"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeCombo):
        text += ":WA_X11NetWmWindowTypeCombo"
    if widget.testAttribute(QtCore.Qt.WA_X11NetWmWindowTypeDND):
        text += ":WA_X11NetWmWindowTypeDND"
    if PYSIDE2:
        if widget.testAttribute(QtCore.Qt.WA_MacFrameworkScaled):
            text += ":WA_MacFrameworkScaled"
    if widget.testAttribute(QtCore.Qt.WA_AcceptTouchEvents):
        text += ":WA_AcceptTouchEvents"
    if widget.testAttribute(QtCore.Qt.WA_TouchPadAcceptSingleTouchEvents):
        text += ":WA_TouchPadAcceptSingleTouchEvents"
    if widget.testAttribute(QtCore.Qt.WA_X11DoNotAcceptFocus):
        text += ":WA_X11DoNotAcceptFocus"
    if widget.testAttribute(QtCore.Qt.WA_AlwaysStackOnTop):
        text += ":WA_AlwaysStackOnTop"
    if widget.testAttribute(QtCore.Qt.WA_ContentsMarginsRespectsSafeArea):
        text += ":WA_ContentsMarginsRespectsSafeArea"
    return text


def log_created_widget(widget: QtWidgets.QWidget, logger: logging.Logger) -> None:
    """
    Logging a created widget
    """
    logger.debug(
        "+=%d : (%s)(%s)",
        id(widget),
        _widget_window_flags_to_string(widget),
        _widget_attributes_to_string(widget),
    )



#!/usr/bin/env python3

"""
Qt model for recipes
"""

import typing

from qtpy import QtCore


from cruiz.settings.managers.recipe import RecipeSettingsReader
import cruiz.globals


class RecipesModel(QtCore.QAbstractTableModel):
    """
    Model representing a Conan recipe
    """

    def __init__(self) -> None:
        super().__init__()
        self._uuids: typing.Optional[typing.List[QtCore.QUuid]] = None

    def set_uuids(self, uuids: typing.List[QtCore.QUuid]) -> None:
        """
        Set the recipe UUIDs for the model
        """
        self.beginResetModel()
        self._uuids = uuids
        self.endResetModel()

    def uuid(self, index: int) -> QtCore.QUuid:
        """
        Get the recipe UUID specified by its index
        """
        assert self._uuids
        return self._uuids[index]

    def rowCount(self, parent) -> int:  # type: ignore
        if parent.isValid():
            return 0
        if self._uuids is None:
            return 0
        return len(self._uuids)

    def columnCount(self, parent) -> int:  # type: ignore
        # pylint: disable=unused-argument
        return 3

    def headerData(self, section, orientation, role):  # type: ignore
        if role == QtCore.Qt.DisplayRole and orientation == QtCore.Qt.Horizontal:
            if section == 0:
                return "Path"
            if section == 1:
                return "Version"
            if section == 2:
                return "Local cache"
        return None

    def data(self, index, role):  # type: ignore
        if role == QtCore.Qt.DisplayRole:
            assert self._uuids
            if index.column() == 0:
                with RecipeSettingsReader.from_uuid(
                    self._uuids[index.row()]
                ) as settings:
                    return settings.path.resolve()
            if index.column() == 1:
                with RecipeSettingsReader.from_uuid(
                    self._uuids[index.row()]
                ) as settings:
                    attributes = settings.attribute_overrides.resolve()
                if "version" in attributes:
                    return attributes["version"]
            if index.column() == 2:
                with RecipeSettingsReader.from_uuid(
                    self._uuids[index.row()]
                ) as settings:
                    return settings.local_cache_name.resolve()
        return None

    def flags(self, index):  # type: ignore
        def_flags = super().flags(index)
        if self._uuids:
            if cruiz.globals.get_main_window().is_recipe_active(
                self._uuids[index.row()]
            ):
                return def_flags & ~QtCore.Qt.ItemIsEnabled  # type: ignore[operator]
        return def_flags



#!/usr/bin/env python3

"""
cruiz: Conan recipe user interface
"""

import importlib.util
import logging
import os
import pathlib
import platform
import sys

from qtpy import QtCore, QtWidgets, PYSIDE2

from cruiz.resourcegeneration import generate_resources


logging.basicConfig(
    level=os.getenv("LOGLEVEL", "WARNING"),
    filename=os.getenv("LOGFILE", None),
)


logger = logging.getLogger(__name__)

# resource generation be invoked before resources and MainWindow are imported
generate_resources()

if PYSIDE2:
    import cruiz.pyside2.resources  # noqa: F401
else:
    import cruiz.pyside6.resources  # noqa: F401

from cruiz.application import Application  # noqa: E402
from cruiz.mainwindow import MainWindow  # noqa: E402
from cruiz.settings.updatesettings import (  # noqa: E402
    update_settings_to_current_version,
)

CONAN_SPEC = importlib.util.find_spec("conans")
if CONAN_SPEC is None:
    QtWidgets.QApplication()
    QtWidgets.QMessageBox.critical(
        None,  # type: ignore
        "Conan unavailable",
        "Unable to locate the conan Python package in the current environment.\n"
        "Use pip install conan[==version].",
    )
    sys.exit(-1)


def _are_resources_out_of_date() -> bool:
    current_dir = pathlib.Path(__file__).parent
    resources_dir = current_dir / "resources"
    if not resources_dir.exists():
        return False
    generated_files = [
        current_dir / "resources.py",
    ]
    if any(not path.exists() for path in generated_files):
        return False
    newest_generated_file = sorted(generated_files, key=os.path.getmtime, reverse=True)[
        0
    ]
    non_ui_resource_files = list(
        set(resources_dir.glob("*")) - set(resources_dir.glob("*.ui"))
    )
    newest_resource_file = sorted(
        non_ui_resource_files, key=os.path.getmtime, reverse=True
    )[0]
    input_mtime = os.path.getmtime(newest_resource_file)
    output_mtime = os.path.getmtime(newest_generated_file)
    return input_mtime > output_mtime


if _are_resources_out_of_date():
    QtWidgets.QApplication()
    QtWidgets.QMessageBox.critical(
        None,  # type: ignore
        "Resources",
        "Resources are out of date.\n" "Please build with python3 setup.py build",
    )
    sys.exit(-1)


QtCore.QCoreApplication.setApplicationName("cruiz")
update_settings_to_current_version()

if platform.system() == "Darwin":
    os.environ["no_proxy"] = "*"


def _warning_message_filter(message: str) -> bool:
    if message.startswith("delivering touch release to same window"):
        return False
    if message.startswith("skipping QEventPoint("):
        return False
    if message.startswith("Populating font family aliases took"):
        return False
    return True


def _message_handler(
    mode: QtCore.QtMsgType, context: QtCore.QMessageLogContext, message: str
) -> None:
    if mode == QtCore.QtMsgType.QtInfoMsg:
        logger.info(message)
    elif mode == QtCore.QtMsgType.QtWarningMsg:
        if _warning_message_filter(message):
            logger.warning(message)
    elif mode == QtCore.QtMsgType.QtCriticalMsg:
        logger.critical(message)
    elif mode == QtCore.QtMsgType.QtFatalMsg:
        raise RuntimeError(message)
    else:
        logger.debug(message)


def main() -> int:
    """
    Entry point
    """
    QtCore.qInstallMessageHandler(_message_handler)

    QtWidgets.QApplication.setAttribute(
        QtCore.Qt.AA_EnableHighDpiScaling, True
    )  # enable highdpi scaling
    QtWidgets.QApplication.setAttribute(
        QtCore.Qt.AA_UseHighDpiPixmaps, True
    )  # use highdpi icons

    app = Application(sys.argv)
    window = MainWindow()
    window.showNormal()
    sys.exit(app.exec_())



#!/usr/bin/env python3

"""
Utility to ensure a default local cache exists
"""

from cruiz.settings.managers.namedlocalcache import (
    AllNamedLocalCacheSettingsReader,
    NamedLocalCacheCreator,
)

from cruiz.constants import DEFAULT_CACHE_NAME


def ensure_default_local_cache() -> None:
    """
    Ensure that the settings has a default local cache
    """
    with AllNamedLocalCacheSettingsReader() as caches:
        need_to_add = DEFAULT_CACHE_NAME not in caches
    if not need_to_add:
        return
    NamedLocalCacheCreator().create(DEFAULT_CACHE_NAME, None, None)



#!/usr/bin/env python3

"""
Conan command parameters
"""

from io import StringIO
import pathlib
import typing

from cruiz.constants import BuildFeatureConstants

from .commonparameters import CommonParameters


# TODO: should a recipe be in here? it is quite a heavyweight Context object though
class CommandParameters(CommonParameters):
    """
    Representation of all the arguments to a command
    """

    def __init__(
        self,
        verb: str,
        worker: typing.Union[
            typing.Callable[[typing.Any, typing.Any], None],
            typing.Callable[[typing.Any, typing.Any, typing.Any], None],
        ],
    ) -> None:
        super().__init__(worker)
        self.verb = verb
        self._recipe_path: typing.Optional[
            pathlib.Path
        ] = None  # TODO: aliased with the recipe, so should we store it?
        self._cwd: typing.Optional[pathlib.Path] = None
        self._profile: typing.Optional[str] = None
        self._common_subdir: typing.Optional[pathlib.PurePosixPath] = None
        self._install_folder: typing.Optional[pathlib.PurePosixPath] = None
        self._imports_folder: typing.Optional[pathlib.PurePosixPath] = None
        self._source_folder: typing.Optional[pathlib.PurePosixPath] = None
        self._build_folder: typing.Optional[pathlib.PurePosixPath] = None
        self._package_folder: typing.Optional[pathlib.PurePosixPath] = None
        self._test_folder: typing.Optional[pathlib.PurePosixPath] = None
        self._name: typing.Optional[str] = None
        self._version: typing.Optional[
            str
        ] = None  # TODO: these are aliased with the recipe, so should we store it?
        self._user: typing.Optional[str] = None
        self._channel: typing.Optional[str] = None
        self._options: typing.Dict[str, str] = {}
        self._force: typing.Optional[bool] = None
        self._package_reference: typing.Optional[str] = None
        self._args: typing.List[str] = []
        self._named_args: typing.Dict[str, str] = {}
        self._time_commands: typing.Optional[bool] = None

    def __str__(self) -> str:
        components = ["conan"]
        components.append(self.verb)
        if self._install_folder:
            components.extend(["-if", str(self._install_folder)])
        if self._imports_folder:
            components.extend(["-imf", str(self._imports_folder)])
        if self._source_folder:
            components.extend(["-sf", str(self._source_folder)])
        if self._build_folder:
            components.extend(["-bf", str(self._build_folder)])
        if self._package_folder:
            components.extend(["-pf", str(self._package_folder)])
        if self._test_folder:
            components.extend(["-tbf", str(self._test_folder)])
        if self._profile:
            components.extend(
                [
                    "-pr",
                    self._profile,
                ]
            )
        for key, value in self._options.items():
            components.extend(
                [
                    "-o",
                    f"{key}={value}",
                ]
            )
        if self._force:
            components.append("-f")
        if self._args:
            components.extend(self._args)
        # no named args
        if self._recipe_path:
            components.append(str(self._recipe_path))
        if self._package_reference:
            components.append(self._package_reference)
        command = " ".join(components)
        return command

    @property
    def cmd_expression(self) -> StringIO:
        """
        Get the expression valid in a CMD batch shell for this command
        """
        expression = StringIO()
        for key, value in self.added_environment.items():
            # internal envvars used in multi-process that don't immediately
            # translate to real envvars as they depend on context and also
            # poking into Conan by monkey patching
            if key.startswith("BuildFeatureConstants"):
                expression.write(f"rem {key}={value}\n")
            # avoid imposing cruiz choices on colour schemes
            if key == "CONAN_COLOR_DARK":
                expression.write(f"rem {key}={value}\n")
        # always use a child shell
        expression.write('cmd /C "')
        if self.cwd:
            expression.write(f"cd {self.cwd} && ")
        for key, value in self.added_environment.items():
            if key.startswith("BuildFeatureConstants"):
                continue
            if key == "CONAN_COLOR_DARK":
                continue
            expression.write(f"set {key}={value} && ")
        for key in self.removed_environment:
            expression.write(f"set {key}= && ")
        expression.write(str(self))
        expression.write('"')
        return expression

    @property
    def bash_expression(self) -> StringIO:
        """
        Get the expression valid in a bash shell for this command
        """
        expression = StringIO()
        for key, value in self.added_environment.items():
            # internal envvars used in multi-process that don't immediately
            # translate to real envvars as they depend on context and also
            # poking into Conan by monkey patching
            if key.startswith("BuildFeatureConstants"):
                expression.write(f"# {key}={value}\n")
            # avoid imposing cruiz choices on colour schemes
            if key == "CONAN_COLOR_DARK":
                expression.write(f"# {key}={value}\n")
        if self.cwd:
            # use a subshell
            expression.write(f"( cd {self.cwd} && ")
        for key, value in self.added_environment.items():
            if key.startswith("BuildFeatureConstants"):
                continue
            if key == "CONAN_COLOR_DARK":
                continue
            expression.write(f"{key}={value} ")
        for key in self.removed_environment:
            expression.write(f"{key}= ")
        expression.write(str(self))
        if self.cwd:
            expression.write(" )")
        return expression

    @property
    def zsh_expression(self) -> StringIO:
        """
        Get the expression valid in a zsh shell for this command
        """
        expression = StringIO()
        expression.write("setopt interactivecomments\n")
        expression.write(self.bash_expression.getvalue())
        return expression

    @property
    def recipe_path(self) -> typing.Optional[pathlib.Path]:
        """
        Get the path of the recipe used in this command.
        May be None.
        """
        return self._recipe_path

    @recipe_path.setter
    def recipe_path(self, value: typing.Optional[pathlib.Path]) -> None:
        self._recipe_path = value

    @property
    def cwd(self) -> typing.Optional[pathlib.Path]:
        """
        Get the current directory used in this command.
        May be None.
        """
        return self._cwd

    @cwd.setter
    def cwd(self, value: pathlib.Path) -> None:
        self._cwd = value

    @property
    def profile(self) -> typing.Optional[str]:
        """
        Get the profile to use in the command against this recipe.
        -pr/--profile switch
        May be None to omit
        """
        return self._profile

    @profile.setter
    def profile(self, value: typing.Optional[str]) -> None:
        self._profile = value

    @property
    def common_subdir(self) -> typing.Optional[pathlib.PurePosixPath]:
        """
        Get the common subdirectory to apply atop of the current working directory.
        May be None to omit
        """
        return self._common_subdir

    @common_subdir.setter
    def common_subdir(self, value: typing.Optional[pathlib.PurePosixPath]) -> None:
        self._common_subdir = value

    @property
    def install_folder(self) -> typing.Optional[pathlib.PurePosixPath]:
        """
        Get the install folder to use in the command against this recipe.
        -if/--install-folder switch
        May be None to omit
        """
        return self._install_folder

    @install_folder.setter
    def install_folder(self, value: typing.Optional[pathlib.PurePosixPath]) -> None:
        self._install_folder = value

    @property
    def imports_folder(self) -> typing.Optional[pathlib.PurePosixPath]:
        """
        Get the imports folder to use in the command against this recipe.
        -imf/--import-folder switch
        May be None to omit
        """
        return self._imports_folder

    @imports_folder.setter
    def imports_folder(self, value: typing.Optional[pathlib.PurePosixPath]) -> None:
        self._imports_folder = value

    @property
    def source_folder(self) -> typing.Optional[pathlib.PurePosixPath]:
        """
        Get the source folder to use in the command against this recipe.
        -sf/--source-folder switch
        May be None to omit
        """
        return self._source_folder

    @source_folder.setter
    def source_folder(self, value: typing.Optional[pathlib.PurePosixPath]) -> None:
        self._source_folder = value

    @property
    def build_folder(self) -> typing.Optional[pathlib.PurePosixPath]:
        """
        Get the build folder to use in the command against this recipe.
        -bf/--build-folder switch
        May be None to omit
        """
        return self._build_folder

    @build_folder.setter
    def build_folder(self, value: typing.Optional[pathlib.PurePosixPath]) -> None:
        self._build_folder = value

    @property
    def package_folder(self) -> typing.Optional[pathlib.PurePosixPath]:
        """
        Get the package folder to use in the command against this recipe.
        -pf/--package-folder switch
        May be None to omit
        """
        return self._package_folder

    @package_folder.setter
    def package_folder(self, value: typing.Optional[pathlib.PurePosixPath]) -> None:
        self._package_folder = value

    @property
    def test_folder(self) -> typing.Optional[pathlib.PurePosixPath]:
        """
        Get the test build folder to use in the command against this recipe.
        -tbf/--test-build-folder switch
        May be None to omit
        """
        return self._test_folder

    @test_folder.setter
    def test_folder(self, value: typing.Optional[pathlib.PurePosixPath]) -> None:
        self._test_folder = value

    @property
    def name(self) -> typing.Optional[str]:
        """
        Get the package name.
        May be None.
        """
        return self._name

    @name.setter
    def name(self, value: typing.Optional[str]) -> None:
        self._name = value

    @property
    def version(self) -> typing.Optional[str]:
        """
        Get the package version.
        May be None.
        """
        return self._version

    @version.setter
    def version(self, value: typing.Optional[str]) -> None:
        self._version = value

    @property
    def user(self) -> typing.Optional[str]:
        """
        Get the package user.
        May be None.
        """
        return self._user

    @user.setter
    def user(self, value: typing.Optional[str]) -> None:
        self._user = value

    @property
    def channel(self) -> typing.Optional[str]:
        """
        Get the package channel.
        May be None.
        """
        return self._channel

    @channel.setter
    def channel(self, value: typing.Optional[str]) -> None:
        self._channel = value

    @property
    def options(self) -> typing.Dict[str, str]:  # TODO: should this be immutable?
        """
        Get the recipe options.
        """
        return self._options

    def add_option(self, package_name: str, key: str, value: str) -> None:
        """
        Add an option key-value pair.
        """
        assert key not in self._options
        self._options[f"{package_name}:{key}"] = value

    @property
    def force(self) -> typing.Optional[bool]:
        """
        Get whether an option is forced.
        May be None.
        """
        return self._force

    @force.setter
    def force(self, value: typing.Optional[bool]) -> None:
        self._force = value

    @property
    def package_reference(self) -> typing.Optional[str]:
        """
        Get the package reference.
        May be None.
        """
        return self._package_reference

    def make_package_reference(self) -> None:
        """
        Make the package reference given name, version, user and channel already set.
        """
        package_reference = StringIO()
        if self._name:
            package_reference.write(f"{self._name}/")
        if self._version:
            package_reference.write(f"{self._version}@")
        if self._user:
            package_reference.write(self._user)
        if self._channel:
            package_reference.write(f"/{self._channel}")
        # in some cases, this can be empty (e.g. export-pkg)
        # but not in others (e.g. test)
        self._package_reference = package_reference.getvalue()

    @property
    def arguments(self) -> typing.List[str]:
        """
        Get the optional arguments passed as a parameter list.
        """
        return self._args

    @property
    def named_arguments(self) -> typing.Dict[str, str]:
        """
        Get the optional named arguments.
        These are not passed on the command line.
        """
        return self._named_args

    @property
    def time_commands(self) -> typing.Optional[bool]:
        """
        Get whether commands are timed.
        May be None.
        """
        return self._time_commands

    @time_commands.setter
    def time_commands(self, value: typing.Optional[bool]) -> None:
        self._time_commands = value

    def set_build_feature(self, feature: BuildFeatureConstants, value: str) -> None:
        """
        Set an optional build feature
        """
        self.added_environment[str(feature)] = value



#!/usr/bin/env python3

"""
Remote browser page
"""

from __future__ import annotations

import html
import pathlib
import shutil
import tarfile
import tempfile
import typing

from qtpy import QtCore, QtGui, QtWidgets, PYSIDE2

if PYSIDE2:
    from qtpy import QtWebEngineWidgets

    QAction = QtWidgets.QAction
else:
    from qtpy import QtWebEngineCore

    QAction = QtGui.QAction

from cruiz.interop.packagebinaryparameters import PackageBinaryParameters

if PYSIDE2:
    from cruiz.pyside2.remote_browser_fileview import Ui_remote_browser_fileview
else:
    from cruiz.pyside6.remote_browser_fileview import Ui_remote_browser_fileview

from cruiz.settings.managers.generalpreferences import GeneralSettingsReader

from .page import Page


class _FileNode:
    def __init__(self, path: str, parent: typing.Optional[_FileNode]):
        self.path = path
        self.parent = parent
        self.children: typing.List[_FileNode] = []
        self.is_file = True
        self.link_target: typing.Optional[str] = None
        self.container: typing.Optional[str] = None
        self.is_container = False
        # am hoping this is reusable on the same tarball but outside the context manager
        self.tar_info: typing.Optional[tarfile.TarInfo] = None

    def add_child(self, path: str) -> _FileNode:
        """
        Add a child path
        """
        find = [x for x in self.children if x.path == path]
        if find:
            assert len(find) == 1
            return find[0]
        node = _FileNode(path, self)
        self.children.append(node)
        return node

    def add_tarball(
        self, directory: pathlib.Path, tarball_basename: str, parent: _FileNode
    ) -> None:
        """
        Adding a tarball containing many children
        """
        tar_node = parent.add_child(tarball_basename)
        tar_node.is_container = True
        tarball_path = directory / tarball_basename
        with tarfile.open(tarball_path, "r") as tar:
            for tarinfo in tar:
                path = pathlib.PurePosixPath(tarinfo.name)
                parent = tar_node
                combined_path = pathlib.PurePosixPath()
                for part in path.parts:
                    combined_path /= part
                    parent = parent.add_child(str(combined_path))
                    parent.is_file = False
                    parent.container = tarball_basename
                if tarinfo.isfile():
                    parent.is_file = True  # the leaf is a file
                    parent.tar_info = tarinfo
                else:
                    if tarinfo.issym():
                        parent.is_file = True  # the leaf is a file
                        parent.link_target = tarinfo.linkname  # and also a symlink
                    else:
                        raise AssertionError(
                            f"Unknown entity type for '{tarinfo.name}'"
                        )

    @property
    def child_index(self) -> int:
        """
        Get the child index of this node in relation to its parent
        """
        if self.parent:
            return self.parent.children.index(self)
        return 0


class _PackageBinaryModel(QtCore.QAbstractItemModel):
    def __init__(self, parent: QtCore.QObject) -> None:
        super().__init__(parent)
        self._root: typing.Optional[_FileNode] = None
        self._parent_object = parent

    def set(
        self,
        results: typing.Optional[typing.List[str]],
        folder: typing.Optional[pathlib.Path],
    ) -> None:
        """
        Set the results against the model
        """
        self.beginResetModel()
        if results:
            assert folder
            self._root = _FileNode("/", None)
            self._root.is_file = False
            for file in results:
                if file.endswith(".tgz"):
                    self._root.add_tarball(folder, file, self._root)
                else:
                    self._root.add_child(file)
        else:
            self._root = None
        self.endResetModel()

    def rowCount(self, parent) -> int:  # type: ignore
        if self._root is None:
            return 0
        if parent.isValid():
            node = parent.internalPointer()
            return len(node.children)
        return 1

    def columnCount(self, parent) -> int:  # type: ignore
        # pylint: disable=unused-argument
        if self._root is None:
            return 0
        return 1

    def parent(self, index) -> QtCore.QModelIndex:  # type: ignore
        if not index.isValid():
            return QtCore.QModelIndex()
        node = index.internalPointer()
        if node.parent is None:
            return QtCore.QModelIndex()
        return super().createIndex(node.parent.child_index, 0, node.parent)

    def index(self, row, column, parent):  # type: ignore
        if not parent.isValid():
            return super().createIndex(row, column, self._root)
        parent_node = parent.internalPointer()
        node = parent_node.children[row]
        return super().createIndex(row, column, node)

    def headerData(self, section, orientation, role):  # type: ignore
        if role == QtCore.Qt.DisplayRole and orientation == QtCore.Qt.Horizontal:
            if section == 0:
                return "Path"
        return None

    def data(self, index, role):  # type: ignore
        node = index.internalPointer()
        if role == QtCore.Qt.DisplayRole:
            if index.column() == 0:
                return pathlib.Path(node.path).name
        if role == QtCore.Qt.ToolTipRole:
            if index.column() == 0:
                if node.is_file and node.link_target:
                    return f"Symbolic link to {node.link_target}"
        if role == QtCore.Qt.DecorationRole:
            if index.column() == 0:
                if node.is_file:
                    if node.link_target:
                        return self._parent_object.style().standardIcon(
                            QtWidgets.QStyle.SP_FileLinkIcon
                        )
                else:
                    return QtWidgets.QFileIconProvider().icon(
                        QtWidgets.QFileIconProvider.Folder
                    )
        return None

    def flags(self, index):  # type: ignore
        default_flags = super().flags(index)
        node = index.internalPointer()
        if node.link_target:
            return default_flags & ~QtCore.Qt.ItemIsEnabled  # type: ignore[operator]
        return default_flags


class _FileViewer(QtWidgets.QDialog):
    def __init__(
        self,
        root: pathlib.Path,
        path: pathlib.Path,
        container: pathlib.Path,
        parent: typing.Optional[QtWidgets.QWidget] = None,
    ) -> None:
        super().__init__(parent)
        self._ui = Ui_remote_browser_fileview()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]
        if PYSIDE2:
            self._ui.fileview.page().settings().setAttribute(
                QtWebEngineWidgets.QWebEngineSettings.LocalContentCanAccessRemoteUrls,
                True,
            )
        else:
            self._ui.fileview.page().settings().setAttribute(
                QtWebEngineCore.QWebEngineSettings.LocalContentCanAccessRemoteUrls, True
            )
        if container:
            html_path = _FileViewer._write_html(path, root / container)
            self.setWindowTitle(f"{path} in {container}")
        else:
            html_path = _FileViewer._write_html(root / path, None)
            self.setWindowTitle(str(path))
        self._ui.fileview.load(QtCore.QUrl.fromLocalFile(str(html_path)))

    @staticmethod
    def _write_html(
        path: pathlib.PurePath, archive: typing.Optional[pathlib.Path]
    ) -> pathlib.Path:
        if archive:
            with tarfile.open(archive, "r") as tar:
                tar.extract(str(path), path=archive.parent)
            with open(archive.parent / path, "rt", encoding="utf-8") as data_file:
                contents = data_file.readlines()
            html_path = archive.parent / path
            html_path = html_path.with_suffix(".html")
        else:
            with open(path, "rt", encoding="utf-8") as data_file:
                contents = data_file.readlines()
            html_path = pathlib.Path(path.with_suffix(".html"))
        html_path.parent.mkdir(parents=True, exist_ok=True)
        with GeneralSettingsReader() as settings:
            use_dark_mode = settings.use_dark_mode.resolve()
        with open(html_path, "wt", encoding="utf-8") as html_file:
            url_start = "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.4.0"
            html_file.write("<html>")
            html_file.write("<head>")
            if use_dark_mode:
                html_file.write(
                    f'<link rel="stylesheet" href="{url_start}/styles/dark.min.css">'
                )
            else:
                html_file.write(
                    f'<link rel="stylesheet" href="{url_start}/styles/default.min.css">'
                )
            html_file.write(f'<script src="{url_start}/highlight.min.js"></script>')
            html_file.write(
                '<script src="https://cdnjs.cloudflare.com/ajax/libs/highlightjs-line-numbers.js/2.8.0/highlightjs-line-numbers.min.js"></script>'  # noqa: E501
            )
            html_file.write("<script>hljs.highlightAll();</script>")
            html_file.write("<script>hljs.initLineNumbersOnLoad();</script>")
            html_file.write("</head>")
            html_file.write('<body style="margin: 0; padding: 0">')
            html_file.write("<pre><code>")
            # with line numbers enabled, we lose leading spaces with html.escape,
            # so put them back manually per line
            for line in contents:
                trimmed = line.lstrip()
                if not trimmed:
                    html_file.write(line)  # honour empty lines
                    continue
                escaped = html.escape(trimmed)
                if len(trimmed) == len(line):
                    html_file.write(escaped)
                    continue
                nbsp_prefix = "&nbsp;" * (len(line) - len(trimmed))
                html_file.write(nbsp_prefix + escaped)
            html_file.write("</code></pre>")
            html_file.write("</body>")
            html_file.write("</html>")
        return html_path


class PackageBinaryPage(Page):
    """
    Remote browser page for displaying package binaries
    """

    def setup(self, self_ui: typing.Any) -> None:
        """
        Setup the UI for the page
        """
        self._base_setup(self_ui, 4)
        self._current_pkgref: typing.Optional[str] = None
        self._artifact_folder: typing.Optional[pathlib.Path] = None
        self._model = _PackageBinaryModel(self)
        self._ui.package_binary.setModel(self._model)
        self._ui.package_binary.customContextMenuRequested.connect(self._on_file_menu)

        self._ui.package_binary.doubleClicked.connect(self._on_prev_dclicked)
        self._ui.pbinary_back.clicked.connect(self._on_back)
        self._ui.pbinary_cancel.clicked.connect(self.on_cancel)
        self._ui.pbinary_restart.clicked.connect(self._on_restart)
        self._ui.pbinary_pkgref.customContextMenuRequested.connect(self._on_pkgref_menu)

    def _enable_progress(self, enable: bool) -> None:
        self._ui.pbinary_progress.setMaximum(0 if enable else 1)
        self._ui.pbinary_buttons.setEnabled(not enable)
        self._ui.pbinary_cancel.setEnabled(enable)

    def showEvent(self, event: QtGui.QShowEvent) -> None:
        # pylint: disable=unused-argument
        pkgref = self._previous_pkgref
        self._ui.pbinary_pkgref.setText(pkgref)

        if self._current_pkgref != pkgref:
            self._artifact_folder = pathlib.Path(tempfile.mkdtemp(suffix="cruiz"))
            self._model.set(None, None)
            self._enable_progress(True)
            self._ui.pbinary_groupbox.setEnabled(False)
            params = PackageBinaryParameters(
                reference=pkgref,
                remote_name=self._ui.remote.currentText(),
                where=self._artifact_folder,
            )
            self._context.get_package_details(params, self._log_details, self._complete)

    def _complete(self, results: typing.Any, exception: typing.Any) -> None:
        self._enable_progress(False)
        if exception:
            self._log_details.stderr(str(exception))
            return
        if results is not None:
            self._model.set(results, self._artifact_folder)
            self._ui.package_binary.expandAll()
            self._ui.pbinary_groupbox.setEnabled(True)
            self._current_pkgref = self._previous_pkgref

    def _on_back(self) -> None:
        if self._revisions_enabled:
            self._open_previous_page()
        else:
            # skip package revisions
            self.parent().setCurrentIndex(self.page_index - 2)

    def _on_prev_dclicked(self, index: QtCore.QModelIndex) -> None:
        node = index.internalPointer()
        if not node.is_file:  # type: ignore
            return
        path = node.path  # type: ignore
        container = node.container  # type: ignore
        try:
            assert self._artifact_folder
            _FileViewer(self._artifact_folder, path, container, self).exec_()
        except UnicodeDecodeError:
            QtWidgets.QMessageBox.critical(
                self,
                "Cannot view file contents",
                f"Unable to interpret file {path} in archive {container} " "as text",
            )

    def on_cancel(self) -> None:
        """
        Called when the user cancels the operation.
        """
        self._context.cancel()
        self._enable_progress(False)

    def _on_restart(self) -> None:
        self._open_start()

    def _on_copy_pkgref_to_clip(self) -> None:
        QtWidgets.QApplication.clipboard().setText(self._ui.pbinary_pkgref.text())

    def _on_file_menu(self, position: QtCore.QPoint) -> None:
        selection = self._ui.package_binary.selectedIndexes()
        assert 1 == len(selection)
        node = selection[0].internalPointer()
        if not node.is_file:
            return
        if node.link_target:
            return
        menu = QtWidgets.QMenu(self)
        save_action = QAction("Save ...", self)
        save_action.triggered.connect(self._on_file_save)
        menu.addAction(save_action)
        menu.exec_(self.sender().mapToGlobal(position))

    def _on_file_save(self) -> None:
        selection = self._ui.package_binary.selectedIndexes()
        assert 1 == len(selection)
        node = selection[0].internalPointer()
        new_path, _ = QtWidgets.QFileDialog.getSaveFileName(
            self, f"Save {node.path} from package", "", ""
        )
        if not new_path:
            return
        if node.container:
            with tarfile.open(self._artifact_folder / node.container, "r") as tar:
                contents_object = tar.extractfile(node.tar_info)
                assert contents_object
                with open(new_path, "wb") as writer:
                    writer.write(contents_object.read())
        else:
            shutil.copyfile(self._artifact_folder / node.path, new_path)



#!/usr/bin/env python3

"""
Settings context managers for general preferences
"""

import platform
import typing

from qtpy import QtGui

from .basesettings import (
    BaseSettings,
    BoolSetting,
    StringSetting,
    ColourSetting,
    ComparableCommonSettings,
    SettingMeta,
)
from .writermixin import _WriterMixin
from .valueclasses import ScalarValue


class GeneralSettings(ComparableCommonSettings):
    """
    Representation of 'general' disk settings
    """

    def __init__(self) -> None:
        super().__init__()

        default_stdout_batching_value = platform.system() == "Windows"

        self._property_meta = {
            "clear_panes": SettingMeta("ClearPanes", BoolSetting, True, ScalarValue),
            "combine_panes": SettingMeta(
                "CombinePanes", BoolSetting, True, ScalarValue
            ),
            "use_stdout_batching": SettingMeta(
                "UseStdoutBatching",
                BoolSetting,
                default_stdout_batching_value,
                ScalarValue,
            ),
            "enable_command_timing": SettingMeta(
                "EnableCommandTimings", BoolSetting, False, ScalarValue
            ),
            "use_dark_mode": SettingMeta("DarkMode", BoolSetting, False, ScalarValue),
            "use_compact_look": SettingMeta(
                "UseCompactLook", BoolSetting, False, ScalarValue
            ),
            "default_recipe_directory": SettingMeta(
                "DefaultRecipeDirectory", StringSetting, None, ScalarValue
            ),
            "default_recipe_editor": SettingMeta(
                "DefaultRecipeEditor", StringSetting, None, ScalarValue
            ),
            "busy_icon_colour": SettingMeta(
                "BusyIconColour",
                ColourSetting,
                QtGui.QColor(QtGui.Qt.black),
                ScalarValue,
            ),
            "found_text_background_colour": SettingMeta(
                "FoundTextBackgroundColour",
                ColourSetting,
                QtGui.QColor(QtGui.Qt.GlobalColor.green),
                ScalarValue,
            ),
            "new_recipe_loading_behaviour": SettingMeta(
                "NewLoadingBehaviour", BoolSetting, False, ScalarValue
            ),
        }

    @property
    def clear_panes(self) -> BoolSetting:
        """
        Get whether output panes are cleared on each command
        """
        return self._get_value_via_meta()

    @clear_panes.setter
    def clear_panes(self, value: bool) -> None:
        self._set_value_via_meta(value)

    @property
    def combine_panes(self) -> BoolSetting:
        """
        Get whether output and error panes are combined
        """
        return self._get_value_via_meta()

    @combine_panes.setter
    def combine_panes(self, value: bool) -> None:
        self._set_value_via_meta(value)

    @property
    def use_stdout_batching(self) -> BoolSetting:
        """
        Get whether standard output and error streamed are batched before displaying
        """
        return self._get_value_via_meta()

    @use_stdout_batching.setter
    def use_stdout_batching(self, value: bool) -> None:
        self._set_value_via_meta(value)

    @property
    def enable_command_timing(self) -> BoolSetting:
        """
        Get whether command wallclock timing is enabled
        """
        return self._get_value_via_meta()

    @enable_command_timing.setter
    def enable_command_timing(self, value: bool) -> None:
        self._set_value_via_meta(value)

    @property
    def use_dark_mode(self) -> BoolSetting:
        """
        Get whether dark mode is enabled
        """
        return self._get_value_via_meta()

    @use_dark_mode.setter
    def use_dark_mode(self, value: bool) -> None:
        self._set_value_via_meta(value)

    @property
    def use_compact_look(self) -> BoolSetting:
        """
        Get whether a compact look is enabled.
        OBSOLETE
        """
        return self._get_value_via_meta()

    @use_compact_look.setter
    def use_compact_look(self, value: bool) -> None:
        self._set_value_via_meta(value)

    @property
    def default_recipe_directory(self) -> StringSetting:
        """
        Get the default recipe directory to look for recipes to load
        """
        return self._get_value_via_meta()

    @default_recipe_directory.setter
    def default_recipe_directory(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def default_recipe_editor(self) -> StringSetting:
        """
        Get the default recipe editor to open recipes in.
        """
        return self._get_value_via_meta()

    @default_recipe_editor.setter
    def default_recipe_editor(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def busy_icon_colour(self) -> ColourSetting:
        """
        Get the colour of the busy icon
        """
        return self._get_value_via_meta()

    @busy_icon_colour.setter
    def busy_icon_colour(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def found_text_background_colour(self) -> ColourSetting:
        """
        Get the background colour of found text.
        """
        return self._get_value_via_meta()

    @found_text_background_colour.setter
    def found_text_background_colour(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def new_recipe_loading_behaviour(self) -> BoolSetting:
        """
        Get whether the new recipe loading behaviour is enabled.
        OSBOLETE
        """
        return self._get_value_via_meta()

    @new_recipe_loading_behaviour.setter
    def new_recipe_loading_behaviour(self, value: str) -> None:
        self._set_value_via_meta(value)


class GeneralSettingsReader:
    """
    Context manager to read from disk settings
    """

    def __init__(self) -> None:
        self.settings = BaseSettings.make_settings()
        self.group = ""

    def __enter__(self) -> GeneralSettings:
        self.settings.beginGroup(self.group)
        self._settings_object = GeneralSettings()
        self._settings_object.settings_reader = self
        return self._settings_object

    def __exit__(
        self, exc_type: typing.Any, exc_value: typing.Any, exc_traceback: typing.Any
    ) -> typing.Any:
        self.settings.endGroup()
        self._settings_object.settings_reader = None
        del self._settings_object
        if exc_type is None:
            pass
        else:
            # propagate exception
            return False


class GeneralSettingsWriter(_WriterMixin):
    """
    Utility class for writing changed settings to disk
    """

    def __init__(self) -> None:
        self._reader_for_writer = GeneralSettingsReader()



#!/usr/bin/env python3

"""
Local workflow folder expression editor dialog
"""

from qtpy import QtWidgets, PYSIDE2

if PYSIDE2:
    from cruiz.pyside2.recipe_local_workflow_expression_editor import (
        Ui_ExpressionEditor,
    )
else:
    from cruiz.pyside6.recipe_local_workflow_expression_editor import (
        Ui_ExpressionEditor,
    )


class ExpressionEditorDialog(QtWidgets.QDialog):
    """
    Dialog for editing expressions for local workflow folders
    """

    def __init__(self, parent: QtWidgets.QWidget) -> None:
        super().__init__(parent)
        self._ui = Ui_ExpressionEditor()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]

        self._ui.expression.textChanged.connect(self._evaluate)
        tokens, _ = parent.tokens()

        self._ui.evaluatedExpression.clear()
        self._ui.nameMacro.setText(tokens["name"])
        self._ui.versionMacro.setText(tokens["version"])
        self._ui.profileMacro.setText(tokens["profile"])
        self._ui.buildtypeMacro.setText(tokens["build_type"])
        self._ui.buildtypelcMacro.setText(tokens["build_type_lc"])

    def _evaluate(self, text: str) -> None:
        try:
            self._ui.evaluatedExpression.setText(self.parent().resolve_expression(text))
        except (ValueError, KeyError):
            self._ui.evaluatedExpression.clear()



#!/usr/bin/env python3

"""
Restore settings to defaults
"""

from cruiz.restart import restart_cruiz

from .basesettings import BaseSettings


def factory_reset() -> None:
    """
    Clear all settings, and restart cruiz.
    """
    settings = BaseSettings.make_settings()
    settings.clear()
    restart_cruiz()



#!/usr/bin/env python3

"""
Settings cleanup
"""

import pathlib

from qtpy import QtCore, QtWidgets

from .basesettings import BaseSettings
from .recentrecipes import RecentRecipeSettingsDeleter, RecentRecipeSettingsWriter
from .recipe import RecipeSettingsDeleter


def sanitise_settings(widget: QtWidgets.QWidget) -> bool:
    """
    Sanitise settings to fix oddities.

    Returns True if the settings were modified, False otherwise.
    """
    incomplete_uuids = {}
    recipe_no_longer_exists_uuid = {}
    recent_recipe_only = []
    with BaseSettings.Group("Recipe") as settings:
        all_uuids = settings.childGroups()
        for uuid in all_uuids:
            with BaseSettings.Group(uuid, settings):
                local_cache = settings.value("LocalCacheName", None)
                if local_cache is None:
                    incomplete_uuids[uuid] = "No local cache name in settings"
                    continue
                profile = settings.value("Profile", None)
                if profile is None:
                    incomplete_uuids[uuid] = "No profile in settings"
                    continue
                path = settings.value("Path", None)
                if path is None:
                    incomplete_uuids[uuid] = "No recipe path in settings"
                    continue
                assert isinstance(path, str)
                path = pathlib.Path(path)
                if not path.exists():
                    recipe_no_longer_exists_uuid[uuid] = path
    with BaseSettings.ReadArray("RecentRecipes", settings) as (_, count):
        for i in range(count):
            settings.setArrayIndex(i)
            recent_uuid = settings.value("UUID", None)
            assert isinstance(recent_uuid, str)
            if recent_uuid in all_uuids:
                all_uuids.remove(recent_uuid)
            else:
                recent_recipe_only.append(recent_uuid)
    cleaned = False
    if incomplete_uuids:
        message = "Incomplete recipe settings:\n"
        for uuid in incomplete_uuids:
            message += f"{uuid}: {incomplete_uuids[uuid]}\n"
        message += "Clean?"
        response = QtWidgets.QMessageBox.question(
            widget,
            "Inconsistent recipe settings",
            message,
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            QtWidgets.QMessageBox.Yes,
        )
        if response == QtWidgets.QMessageBox.Yes:
            for uuid in incomplete_uuids:
                real_uuid = QtCore.QUuid(uuid)
                RecipeSettingsDeleter().delete(real_uuid)
                RecentRecipeSettingsDeleter().delete(real_uuid)
            cleaned = True
    if recipe_no_longer_exists_uuid:
        message = "Recipe no longer exists:\n"
        for _, recipe_path in recipe_no_longer_exists_uuid.items():
            message += f"{recipe_path}\n"
        message += "Clean?"
        response = QtWidgets.QMessageBox.question(
            widget,
            "Inconsistent recipe settings",
            message,
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            QtWidgets.QMessageBox.Yes,
        )
        if response == QtWidgets.QMessageBox.Yes:
            for uuid, _ in recipe_no_longer_exists_uuid.items():
                real_uuid = QtCore.QUuid(uuid)
                RecipeSettingsDeleter().delete(real_uuid)
                RecentRecipeSettingsDeleter().delete(real_uuid)
            cleaned = True
    if recent_recipe_only:
        message = "Missing settings for UUIDs:\n"
        for uuid in recent_recipe_only:
            message += f"{uuid}\n"
        message += "Clean?"
        response = QtWidgets.QMessageBox.question(
            widget,
            "Inconsistent recipe settings",
            message,
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            QtWidgets.QMessageBox.Yes,
        )
        if response == QtWidgets.QMessageBox.Yes:
            for uuid in recent_recipe_only:
                real_uuid = QtCore.QUuid(uuid)
                RecentRecipeSettingsDeleter().delete(real_uuid)
            cleaned = True
    if all_uuids:
        # all_uuids are now those recipe settings without a recent recipe UUID
        message = "Recent recipe UUIDs that don't have settings:\n"
        for uuid in all_uuids:
            message += f"{uuid}\n"
        message += "Add?"
        response = QtWidgets.QMessageBox.question(
            widget,
            "Inconsistent recipe settings",
            message,
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            QtWidgets.QMessageBox.Yes,
        )
        if response == QtWidgets.QMessageBox.Yes:
            for uuid in all_uuids:
                real_uuid = QtCore.QUuid(uuid)
                RecentRecipeSettingsWriter().make_current(real_uuid)
            cleaned = True
    return cleaned



#!/usr/bin/env python3

"""
Conan manage local cache widgets
"""

from .newlocalcachewizard import NewLocalCacheWizard
from .installconfigdialog import InstallConfigDialog
from .movelocalcachedialog import MoveLocalCacheDialog
from .addremotedialog import AddRemoteDialog
from .progressdialogs import RemoveLocksDialog, RemoveAllPackagesDialog
from .addextraprofiledirectorydialog import AddExtraProfileDirectoryDialog
from .addenvironmentdialog import AddEnvironmentDialog
from .runconancommanddialog import RunConanCommandDialog
from .removeenvironmentdialog import RemoveEnvironmentDialog



#!/usr/bin/env python3

"""
Package id parameters
"""

import typing

from .commonparameters import CommonParameters


class PackageIdParameters(CommonParameters):
    """
    Representation of all the arguments to get package ids
    """

    def __init__(self, **args: typing.Any) -> None:
        import cruiz.workers.packagedetails

        super().__init__(cruiz.workers.packagedetails.invoke)
        for k, v in args.items():
            self.__setattr__(k, v)



#!/usr/bin/env python3

"""
Remote browser page
"""

import typing

from qtpy import QtCore, QtGui

from cruiz.commands.context import ConanConfigBoolean

from cruiz.interop.searchrecipesparameters import SearchRecipesParameters

from cruiz.settings.managers.namedlocalcache import AllNamedLocalCacheSettingsReader

from cruiz.widgets.util import BlockSignals

from .page import Page


class _PackageReferenceModel(QtCore.QAbstractListModel):
    def __init__(self) -> None:
        super().__init__()
        self._list: typing.List[str] = []

    def clear(self) -> None:
        """
        Clear the model
        """
        self.beginResetModel()
        self._list = []
        self.endResetModel()

    def add(self, text: str) -> None:
        """
        Add a package reference to the model
        """
        count = len(self._list)
        self.beginInsertRows(QtCore.QModelIndex(), count, count + 1)
        self._list.append(text)
        self.endInsertRows()

    def rowCount(self, parent) -> int:  # type: ignore
        if parent.isValid():
            return 0
        if not self._list:
            return 0
        return len(self._list)

    def data(self, index, role) -> typing.Any:  # type: ignore
        if role == QtCore.Qt.DisplayRole:
            return self._list[index.row()]
        return None

    def flags(self, index):  # type: ignore
        default_flags = super().flags(index)
        if "->" in self._list[index.row()]:
            return default_flags & ~QtCore.Qt.ItemIsEnabled  # type: ignore[operator]
        return default_flags


class _PackageSearchValidator(QtGui.QValidator):
    """
    Validate the input given to the package search
    """

    def validate(self, input_to_validate: str, pos: int) -> QtGui.QValidator.State:
        # pylint: disable=unused-argument
        if not input_to_validate:
            return QtGui.QValidator.Invalid
        return QtGui.QValidator.Acceptable


class PackageReferencePage(Page):
    """
    Remote browser page for finding package references
    """

    def setup(self, self_ui: typing.Any) -> None:
        """
        Setup the UI for the page
        """
        self._base_setup(self_ui, 0)

        self._revs_enabled = False
        self._model = _PackageReferenceModel()
        self._ui.package_references.setModel(self._model)
        self._ui.package_references.customContextMenuRequested.connect(
            self._on_selected_pkgref_menu
        )

        self._ui.local_cache_name.currentTextChanged.connect(self.on_local_cache_change)
        self._ui.remote.currentTextChanged.connect(self._on_remote_change)

        self._ui.search_pattern.setPlaceholderText("e.g. *xyz*")
        self._ui.search_pattern.setValidator(_PackageSearchValidator(self))
        self._ui.search_pattern.inputRejected.connect(self._pattern_incorrect)
        self._ui.search_pattern.returnPressed.connect(self._search)

        self._ui.package_references.doubleClicked.connect(self._on_pkgref_dclicked)

        self._ui.pkgref_cancel.clicked.connect(self.on_cancel)

        self.refresh_local_cache_names()

    def refresh_local_cache_names(self) -> None:
        """
        Refresh the UI for current local cache names.
        """
        with BlockSignals(self._ui.local_cache_name) as blocked_widget:
            blocked_widget.clear()
            with AllNamedLocalCacheSettingsReader() as cache_names:
                blocked_widget.addItems(cache_names)
            self._ui.local_cache_name.setCurrentIndex(-1)
        self._ui.local_cache_name.setCurrentIndex(0)

    @property
    def package_reference(self) -> str:
        """
        Get the package reference selected on this page
        """
        selection = self._ui.package_references.selectedIndexes()
        assert 1 == len(selection)
        return self._model.data(selection[0], QtCore.Qt.DisplayRole)

    def on_local_cache_change(self, text: str) -> None:
        """
        Called when which local cache has been selected has changed
        """
        self._context.change_cache(text)
        with BlockSignals(self._ui.remote) as blocked_widget:
            blocked_widget.clear()
            for remote in self._context.get_remotes_list():
                blocked_widget.addItem(remote.name)
            blocked_widget.setCurrentIndex(-1)
        self._ui.remote.setCurrentIndex(0)
        self._revs_enabled = self._context.get_boolean_config(
            ConanConfigBoolean.REVISIONS, False
        )
        self._ui.revisions.setChecked(self._revs_enabled)

    def _on_remote_change(self, text: str) -> None:
        # pylint: disable=unused-argument
        self._model.clear()

    def _pattern_incorrect(self) -> None:
        with BlockSignals(self._ui.search_pattern) as blocked_widget:
            blocked_widget.setText("")  # clear doesn't actually clear

    def _enable_progress(self, enable: bool) -> None:
        self._ui.pkgref_ui_group.setEnabled(not enable)
        self._ui.pkgref_progress.setMaximum(0 if enable else 1)
        self._ui.pkgref_cancel.setEnabled(enable)

    def _search(self) -> None:
        self._model.clear()
        self._ui.pkgref_groupbox.setTitle("0 package references found")
        self._enable_progress(True)
        self._ui.pkgref_groupbox.setEnabled(False)
        params = SearchRecipesParameters(
            remote_name=self._ui.remote.currentText(),
            pattern=self._ui.search_pattern.text(),
            case_sensitive=True,
            alias_aware=self._ui.alias_aware.isChecked(),
        )
        self._context.get_package_details(params, self._log_details, self._complete)

    def _complete(self, results: typing.Any, exception: typing.Any) -> None:
        self._enable_progress(False)
        if exception:
            self._ui.pkgref_groupbox.setTitle("Error finding package references")
            self._log_details.stderr(str(exception))
            return
        if results is not None:
            for pkgref, alias in results:
                if alias:
                    self._model.add(f"{pkgref} -> {alias}")
                else:
                    self._model.add(pkgref)
            self._ui.pkgref_groupbox.setTitle(
                f"{len(results)} package references found"
            )
            self._ui.pkgref_groupbox.setEnabled(True)

    def _on_pkgref_dclicked(self, index: QtCore.QModelIndex) -> None:
        # pylint: disable=unused-argument
        if self._revisions_enabled:
            self._open_next_page()
        else:
            # skip recipe revisions
            self.parent().setCurrentIndex(self.page_index + 2)

    def on_cancel(self) -> None:
        """
        Called when the user cancels the operation.
        """
        self._context.cancel()
        self._enable_progress(False)

    def invalidate(self) -> None:
        """
        Invalidate the package reference search.
        """
        self._model.clear()
        self._ui.search_pattern.clear()



#!/usr/bin/env python3

"""
Dialog for adding an environment variable
"""

from dataclasses import dataclass

from qtpy import QtCore, QtGui, QtWidgets, PYSIDE2

from cruiz.commands.context import ConanContext

if PYSIDE2:
    from cruiz.pyside2.local_cache_add_environment import Ui_AddEnvironmentDialog

    QAction = QtWidgets.QAction
else:
    from cruiz.pyside6.local_cache_add_environment import Ui_AddEnvironmentDialog

    QAction = QtGui.QAction


@dataclass
class KeyValuePair:
    """
    Representation of a key-value pair
    """

    key: str
    value: str


class AddEnvironmentDialog(QtWidgets.QDialog):
    """
    Dialog for adding an environment key-value pair to the local cache.
    """

    def __init__(self, context: ConanContext, parent: QtWidgets.QWidget) -> None:
        super().__init__(parent)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self._ui = Ui_AddEnvironmentDialog()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]
        conan_environment_menu = QtWidgets.QMenu("Conan environment variables", self)
        for key, _ in context.get_conan_config_environment_variables().items():
            key_action = QAction(key, self)
            key_action.triggered.connect(self._set_name)
            conan_environment_menu.addAction(key_action)
        conan_environment_menu.addSeparator()
        # TODO: CONAN_V2_MODE is obsolete
        conan_v2_mode_action = QAction("CONAN_V2_MODE", self)
        conan_v2_mode_action.triggered.connect(self._set_name)  # type: ignore
        conan_environment_menu.addAction(conan_v2_mode_action)
        self._ui.name.set_custom_menu(conan_environment_menu)
        self._ui.name.textChanged.connect(self._updated)
        self._ui.value.textChanged.connect(self._updated)
        self._ui.buttonBox.button(QtWidgets.QDialogButtonBox.Ok).setEnabled(False)
        self._name: str = ""
        self._value: str = ""

    def _set_name(self) -> None:
        self._ui.name.setText(self.sender().text())

    def _updated(self) -> None:
        self._ui.buttonBox.button(QtWidgets.QDialogButtonBox.Ok).setEnabled(
            bool(self._ui.name.text() and self._ui.value.text())
        )

    def accept(self) -> None:
        self._name = self._ui.name.text()
        self._value = self._ui.value.text()
        super().accept()

    @property
    def environment_variable(self) -> KeyValuePair:
        """
        Get the environment key-value pair just added.
        """
        return KeyValuePair(self._name, self._value)



#!/usr/bin/env python3

"""
Child process commands
"""

from __future__ import annotations

import multiprocessing

from cruiz.interop.packageidparameters import PackageIdParameters

from .utils import worker
from .utils.message import Message, Success


def invoke(queue: multiprocessing.Queue[Message], params: PackageIdParameters) -> None:
    """
    Equivalent to

    'conan search -r <remote_name> <reference>'

    PackageIdParameters has dynamic attributes.
    """
    with worker.ConanWorker(queue, params) as api:
        result = api.search_packages(
            params.reference,  # type: ignore
            remote_name=params.remote_name,  # type: ignore
        )
        results_list = result["results"][0]["items"][0]["packages"]

        queue.put(Success(results_list or None))



#!/usr/bin/env python3

"""
Settings context managers for shortcuts
"""

import typing

from .basesettings import (
    BaseSettings,
    StringSetting,
    ComparableCommonSettings,
    SettingMeta,
)
from .writermixin import _WriterMixin
from .valueclasses import ScalarValue


class ShortcutSettings(ComparableCommonSettings):
    """
    Settings for keyboard shortcuts
    """

    def __init__(self) -> None:
        self._property_meta = {
            "conan_create": SettingMeta(
                "Conan/Create", StringSetting, "Alt+C", ScalarValue
            ),
            "conan_create_updates": SettingMeta(
                "Conan/CreateUpdate", StringSetting, "Shift+Alt+C", ScalarValue
            ),
            "conan_imports": SettingMeta(
                "Conan/Imports", StringSetting, "Alt+M", ScalarValue
            ),
            "conan_install": SettingMeta(
                "Conan/Install", StringSetting, "Alt+I", ScalarValue
            ),
            "conan_install_updates": SettingMeta(
                "Conan/InstallUpdate", StringSetting, "Shift+Alt+I", ScalarValue
            ),
            "conan_source": SettingMeta(
                "Conan/Source", StringSetting, "Alt+S", ScalarValue
            ),
            "conan_build": SettingMeta(
                "Conan/Build", StringSetting, "Alt+B", ScalarValue
            ),
            "conan_package": SettingMeta(
                "Conan/Package", StringSetting, "Alt+P", ScalarValue
            ),
            "conan_export_package": SettingMeta(
                "Conan/ExportPackage", StringSetting, "Alt+X", ScalarValue
            ),
            "conan_test_package": SettingMeta(
                "Conan/TestPackage", StringSetting, "Alt+T", ScalarValue
            ),
            "conan_remove_package": SettingMeta(
                "Conan/RemovePackage", StringSetting, "Alt+-", ScalarValue
            ),
            "cancel": SettingMeta("Cancel", StringSetting, "Meta+C", ScalarValue),
            "cmake_build_tool": SettingMeta(
                "CMake/RunBuildTool", StringSetting, "Meta+B", ScalarValue
            ),
            "cmake_build_tool_verbose": SettingMeta(
                "CMake/RunBuildToolVerbose", StringSetting, "Shift+Meta+B", ScalarValue
            ),
            "delete_cmake_cache": SettingMeta(
                "CMake/DeleteCacheFile", StringSetting, "Meta+X", ScalarValue
            ),
        }

    @property
    def conan_create(self) -> StringSetting:
        """
        Get the shortcut for 'conan create'
        """
        return self._get_value_via_meta()

    @conan_create.setter
    def conan_create(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_create_updates(self) -> StringSetting:
        """
        Get the shortcut for 'conan create -u'
        """
        return self._get_value_via_meta()

    @conan_create_updates.setter
    def conan_create_updates(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_imports(self) -> StringSetting:
        """
        Get the shortcut for 'conan imports'
        """
        return self._get_value_via_meta()

    @conan_imports.setter
    def conan_imports(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_install(self) -> StringSetting:
        """
        Get the shortcut for 'conan install'
        """
        return self._get_value_via_meta()

    @conan_install.setter
    def conan_install(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_install_updates(self) -> StringSetting:
        """
        Get the shortcut for 'conan install -u'
        """
        return self._get_value_via_meta()

    @conan_install_updates.setter
    def conan_install_updates(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_source(self) -> StringSetting:
        """
        Get the shortcut for 'conan source'
        """
        return self._get_value_via_meta()

    @conan_source.setter
    def conan_source(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_build(self) -> StringSetting:
        """
        Get the shortcut for 'conan build'
        """
        return self._get_value_via_meta()

    @conan_build.setter
    def conan_build(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_package(self) -> StringSetting:
        """
        Get the shortcut for 'conan package'
        """
        return self._get_value_via_meta()

    @conan_package.setter
    def conan_package(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_export_package(self) -> StringSetting:
        """
        Get the shortcut for 'conan export-pkg'
        """
        return self._get_value_via_meta()

    @conan_export_package.setter
    def conan_export_package(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_test_package(self) -> StringSetting:
        """
        Get the shortcut for 'conan test'
        """
        return self._get_value_via_meta()

    @conan_test_package.setter
    def conan_test_package(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def conan_remove_package(self) -> StringSetting:
        """
        Get the shortcut for 'conan remove'
        """
        return self._get_value_via_meta()

    @conan_remove_package.setter
    def conan_remove_package(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def cancel(self) -> StringSetting:
        """
        Get the shortcut for cancelling the current command
        """
        return self._get_value_via_meta()

    @cancel.setter
    def cancel(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def cmake_build_tool(self) -> StringSetting:
        """
        Get the shortcut for running the CMake build tool
        """
        return self._get_value_via_meta()

    @cmake_build_tool.setter
    def cmake_build_tool(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def cmake_build_tool_verbose(self) -> StringSetting:
        """
        Get the shortcut for CMake build tool in verbose
        """
        return self._get_value_via_meta()

    @cmake_build_tool_verbose.setter
    def cmake_build_tool_verbose(self, value: str) -> None:
        self._set_value_via_meta(value)

    @property
    def delete_cmake_cache(self) -> StringSetting:
        """
        Get the shortcut for deleting the CMake cache
        """
        return self._get_value_via_meta()

    @delete_cmake_cache.setter
    def delete_cmake_cache(self, value: str) -> None:
        self._set_value_via_meta(value)


class ShortcutSettingsReader:
    """
    Context manager to read shortcuts from disk
    """

    def __init__(self) -> None:
        self.settings = BaseSettings.make_settings()
        self.group = "Shortcut"

    def __enter__(self) -> ShortcutSettings:
        self.settings.beginGroup(self.group)
        self._settings_object = ShortcutSettings()
        self._settings_object.settings_reader = self
        return self._settings_object

    def __exit__(
        self, exc_type: typing.Any, exc_value: typing.Any, exc_traceback: typing.Any
    ) -> typing.Any:
        self.settings.endGroup()
        self._settings_object.settings_reader = None
        del self._settings_object
        if exc_type is None:
            pass
        else:
            # propagate exception
            return False


class ShortcutSettingsWriter(_WriterMixin):
    """
    Context manager to write shortcuts to disk
    """

    def __init__(self) -> None:
        self._reader_for_writer = ShortcutSettingsReader()



#!/usr/bin/env python3

"""
Dialog for running a Conan command in a local cache.
"""

import typing

from qtpy import QtCore, QtWidgets, PYSIDE2

from cruiz.commands.context import ConanContext, LogDetails
from cruiz.interop.commandparameters import CommandParameters

if PYSIDE2:
    from cruiz.pyside2.local_cache_run_command_dialog import Ui_RunConanCommandDialog
else:
    from cruiz.pyside6.local_cache_run_command_dialog import Ui_RunConanCommandDialog

import cruiz.workers.arbitrary


class RunConanCommandDialog(QtWidgets.QDialog):
    """
    Dialog for adding an environment key-value pair to the local cache.
    """

    def __init__(self, context_name: str, parent: QtWidgets.QWidget) -> None:
        super().__init__(parent)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self._ui = Ui_RunConanCommandDialog()
        self._ui.setupUi(self)  # type: ignore[no-untyped-call]
        self._log_details = LogDetails(self._ui.log, self._ui.log, True, False, None)
        self._context = ConanContext(context_name, self._log_details)
        self._ui.arguments.textChanged.connect(self._on_arguments_changed)
        self._ui.run.clicked.connect(self._on_run)
        self._ui.run.setEnabled(False)

    def done(self, result: int) -> None:
        self._context.close()
        super().done(result)

    def _on_arguments_changed(self, text: str) -> None:
        self._ui.run.setEnabled(bool(text))

    def _on_run(self) -> None:
        args = self._ui.arguments.text().split()
        params = CommandParameters(args[0], cruiz.workers.arbitrary.invoke)
        params.arguments.extend(args[1:])
        self._ui.arguments.setEnabled(False)
        self._ui.run.setEnabled(False)
        self._context.run_any_command(params, self._on_run_complete)

    def _on_run_complete(self, payload: typing.Any, exception: typing.Any) -> None:
        # payload is always None
        self._ui.arguments.setEnabled(True)
        self._ui.run.setEnabled(True)
        if exception:
            self._log_details.stderr("<font color='red'>Failed</font><br>")
        else:
            self._log_details.stdout("<font color='green'>Succeeded</font><br>")
